@@ -1,6 +1,6 @@
 /*
     Drumstick MIDI File Player Multiplatform Program
-    Copyright (C) 2006-2021, Pedro Lopez-Cabanillas <plcl@users.sf.net>
+    Copyright (C) 2006-2022, Pedro Lopez-Cabanillas <plcl@users.sf.net>
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -16,7 +16,6 @@
     along with this program. If not, see <http://www.gnu.org/licenses/>.
 */
 
-#include <QDebug>
 #include <QFileDialog>
 #include <QToolTip>
 #include <QMessageBox>
@@ -27,6 +26,13 @@
 #include <QMimeData>
 #include <QtMath>
 #include <QActionGroup>
+#include <QStandardPaths>
+#include <QDesktopServices>
+#if QT_VERSION < QT_VERSION_CHECK(5,14,0)
+#include <QDesktopWidget>
+#else
+#include <QScreen>
+#endif
 
 #include <drumstick/settingsfactory.h>
 #include "guiplayer.h"
@@ -38,6 +44,8 @@
 #include "pianola.h"
 #include "settings.h"
 #include "prefsdialog.h"
+#include "loopdialog.h"
+#include "helpwindow.h"
 
 using namespace drumstick::rt;
 using namespace drumstick::widgets;
@@ -56,30 +64,11 @@ GUIPlayer::GUIPlayer(QWidget *parent)
     m_midiOut(nullptr),
     m_player(nullptr),
     m_ui(new Ui::GUIPlayerClass),
-    m_pd(nullptr)
+    m_pd(nullptr),
+    m_currentLang(nullptr)
 {
-    m_trq = new QTranslator(this);
-    m_trp = new QTranslator(this);
-    m_trl = new QTranslator(this);
-    QString lang = Settings::instance()->language();
-    if (!m_trq->load("qt_" + lang, Settings::systemLocales()) && !lang.startsWith("en")) {
-        qWarning() << "Failure loading Qt5 system translations for" << lang
-                   << "from" << Settings::systemLocales();
-    }
-    if (!m_trp->load("dmidiplayer_" + lang, Settings::localeDirectory()) && !lang.startsWith("en")) {
-        qWarning() << "Failure loading application translations for" << lang
-                   << "from" << Settings::localeDirectory();
-    }
-    if (!m_trl->load("drumstick-widgets_" + lang, Settings::drumstickLocales()) && !lang.startsWith("en")) {
-        qWarning() << "Failure loading widgets library translations for" << lang
-                   << "from" << Settings::drumstickLocales();
-    }
-    QCoreApplication::installTranslator(m_trq);
-    QCoreApplication::installTranslator(m_trp);
-    QCoreApplication::installTranslator(m_trl);
-    Settings::instance()->retranslatePalettes();
-	m_ui->setupUi(this);
-	setAcceptDrops(true);
+    m_ui->setupUi(this);
+    setAcceptDrops(true);
     connect(m_ui->actionAbout, &QAction::triggered, this, &GUIPlayer::about);
     connect(m_ui->actionAboutQt, &QAction::triggered, qApp, &QApplication::aboutQt);
     connect(m_ui->actionPlay, &QAction::triggered, this, &GUIPlayer::play);
@@ -111,6 +100,12 @@ GUIPlayer::GUIPlayer(QWidget *parent)
     connect(m_ui->actionForward, &QAction::triggered, this, &GUIPlayer::forward);
     connect(m_ui->actionBackward, &QAction::triggered, this, &GUIPlayer::backward);
     connect(m_ui->actionJump, &QAction::triggered, this, &GUIPlayer::jump);
+    connect(m_ui->actionLoop, &QAction::triggered, this, &GUIPlayer::loop);
+    connect(m_ui->actionContents, &QAction::triggered, this, &GUIPlayer::slotHelp);
+    connect(m_ui->actionWebSite, &QAction::triggered, this, &GUIPlayer::slotOpenWebSite);
+    connect(m_ui->actionLoadSongSettings, &QAction::triggered, this, &GUIPlayer::slotLoadSongSettings);
+    connect(m_ui->actionSaveSongSettings, &QAction::triggered, this, &GUIPlayer::slotSaveSongSettings);
+    connect(m_ui->actionSearch, &QAction::triggered, this, &GUIPlayer::slotSearch);
 
     m_ui->actionPlay->setShortcut( Qt::Key_MediaPlay );
     m_ui->actionStop->setShortcut( Qt::Key_MediaStop );
@@ -161,10 +156,10 @@ GUIPlayer::GUIPlayer(QWidget *parent)
     connect(m_player, &SequencePlayer::midiProgram, m_channels, &Channels::slotPatch);
     connect(m_channels, &Channels::name, m_pianola, &Pianola::slotLabel);
 
-    connect(m_channels, &Channels::mute, m_player, &SequencePlayer::setMuted);
-    connect(m_channels, &Channels::volume, m_player, &SequencePlayer::setVolume);
-    connect(m_channels, &Channels::lock, m_player, &SequencePlayer::setLocked);
-    connect(m_channels, &Channels::patch, m_player, &SequencePlayer::setPatch);
+    connect(m_channels, &Channels::mute, this, &GUIPlayer::slotMuted);
+    connect(m_channels, &Channels::volume, this, &GUIPlayer::slotVolume);
+    connect(m_channels, &Channels::lock, this, &GUIPlayer::slotLocked);
+    connect(m_channels, &Channels::patch, this, &GUIPlayer::slotPatch);
 
     m_lyrics = new Lyrics(this);
     connect(m_lyrics, &Lyrics::closed, this, &GUIPlayer::slotLyricsClosed);
@@ -177,15 +172,24 @@ GUIPlayer::GUIPlayer(QWidget *parent)
     updNavButtons();
 
     m_toolbarEditor = new ToolBarEditDialog(m_ui->toolBar, this);
+    m_helpWindow = new HelpWindow(this);
+
+    m_ui->frame->setPalette(QPalette(QColor(0x0f,0x0f,0x0f)));
+    QFont lblTimeFont = QApplication::font();
+    lblTimeFont.setPointSize(36);
+    m_ui->lblTime->setFont(lblTimeFont);
 
     try {
         BackendManager man;
         man.refresh(Settings::instance()->settingsMap());
         QList<MIDIOutput*> outputs = man.availableOutputs();
+        if (outputs.isEmpty()) {
+            qWarning() << "MIDI OUT drivers missing. Perhaps you need to set a DRUMSTICKRT environment variable?";
+        }
 
         m_midiOut = man.findOutput(Settings::instance()->lastOutputBackend());
         if (m_midiOut == nullptr) {
-            qWarning() << "MIDI OUT driver not found";
+            qWarning() << "MIDI OUT driver not found. Perhaps you need to set a DRUMSTICKRT environment variable?";
         }
 
         SettingsFactory settings;
@@ -198,7 +202,9 @@ GUIPlayer::GUIPlayer(QWidget *parent)
             auto outConnections = m_midiOut->connections(Settings::instance()->advanced());
             auto itr = std::find_if(outConnections.constBegin(), outConnections.constEnd(), [lastConn](const MIDIConnection& c){return c.first == lastConn;});
             if (itr == outConnections.constEnd()) {
-                conn = outConnections.first();
+                if (!outConnections.isEmpty()) {
+                    conn = outConnections.first();
+                }
             } else {
                 conn = (*itr);
             }
@@ -219,7 +225,7 @@ GUIPlayer::GUIPlayer(QWidget *parent)
             m_player->setPort(m_midiOut);
             connectOutPort();
         } else {
-            qWarning() << "MIDI Output is not connected";
+            qWarning() << "MIDI Output is not connected. Playback is not possible.";
         }
 
         tempoReset();
@@ -277,7 +283,8 @@ void GUIPlayer::updateState(PlayerState newState)
         m_ui->actionForward->setEnabled(false);
         m_ui->actionBackward->setEnabled(false);
         m_ui->actionJump->setEnabled(false);
-        statusBar()->showMessage("Please, load a song");
+        m_ui->actionLoop->setEnabled(false);
+        statusBar()->showMessage(tr("Please, load a song"));
         break;
     case PlayingState:
         m_ui->actionPlay->setEnabled(false);
@@ -286,7 +293,8 @@ void GUIPlayer::updateState(PlayerState newState)
         m_ui->actionForward->setEnabled(true);
         m_ui->actionBackward->setEnabled(true);
         m_ui->actionJump->setEnabled(true);
-        statusBar()->showMessage("Playing");
+        m_ui->actionLoop->setEnabled(true);
+        statusBar()->showMessage(tr("Playing"));
         break;
     case PausedState:
         m_ui->actionPlay->setEnabled(false);
@@ -294,7 +302,8 @@ void GUIPlayer::updateState(PlayerState newState)
         m_ui->actionForward->setEnabled(true);
         m_ui->actionBackward->setEnabled(true);
         m_ui->actionJump->setEnabled(true);
-        statusBar()->showMessage("Paused");
+        m_ui->actionLoop->setEnabled(true);
+        statusBar()->showMessage(tr("Paused"));
         break;
     case StoppedState:
         m_ui->actionPause->setChecked(false);
@@ -304,12 +313,13 @@ void GUIPlayer::updateState(PlayerState newState)
         m_ui->actionForward->setEnabled(true);
         m_ui->actionBackward->setEnabled(true);
         m_ui->actionJump->setEnabled(true);
+        m_ui->actionLoop->setEnabled(true);
         updateTimeLabel(0);
         m_ui->positionSlider->setValue(0);
-        statusBar()->showMessage("Stopped");
+        statusBar()->showMessage(tr("Stopped"));
         break;
     default:
-        statusBar()->showMessage("Not initialized");
+        statusBar()->showMessage(tr("Not initialized"));
         break;
     }
     m_state = newState;
@@ -323,6 +333,7 @@ void GUIPlayer::play()
         if (m_player->getPosition() == 0) {
             m_player->sendResetMessage();
             m_player->resetControllers();
+            m_player->resetPrograms();
             m_player->sendVolumeEvents();
         }
         m_playerThread.start(QThread::HighPriority);
@@ -356,7 +367,7 @@ void GUIPlayer::stop()
 void GUIPlayer::progressDialogInit(const QString& type, int max)
 {
     m_pd = new QProgressDialog(0, 0, 0, max, this);
-    m_pd->setWindowTitle(QString("Loading %1 file...").arg(type));
+    m_pd->setWindowTitle(tr("Loading %1 file...").arg(type));
     m_pd->setMinimumDuration(0); // 1000
     m_pd->setMaximum(max);
     m_pd->setValue(0);
@@ -383,6 +394,8 @@ void GUIPlayer::openFile(const QString& fileName)
         QString ext = finfo.suffix().toLower();
         if (ext == "wrk") {
             progressDialogInit("Cakewalk", finfo.size());
+        } else if (ext == "rmi") {
+            progressDialogInit("RIFF MIDI", finfo.size());
         } else if (ext == "mid" || ext == "midi" || ext == "kar") {
             progressDialogInit("MIDI", finfo.size());
         }
@@ -423,10 +436,25 @@ void GUIPlayer::openFile(const QString& fileName)
                 m_lyrics->initSong( m_player->song() );
             }
 
+            if (Settings::instance()->autoSongSettings()) {
+                slotLoadSongSettings();
+            }
+
             if ( Settings::instance()->getAutoPlay() ) {
                 QTimer::singleShot(0, this, &GUIPlayer::play);
             }
         }
+    } else {
+        QMessageBox::warning(this, QSTR_APPNAME, tr("file %1 couldn't be opened").arg(finfo.fileName()));
+    }
+}
+
+void GUIPlayer::openFileList(const QStringList &fileNames)
+{
+    m_playList->clearPlayList();
+    m_playList->setItems(fileNames);
+    if (m_playList->selectFirstItem()) {
+        openFile(m_playList->currentItem());
     }
 }
 
@@ -434,9 +462,10 @@ void GUIPlayer::open()
 {
     QString fileName = QFileDialog::getOpenFileName(this,
           tr("Open MIDI File"), Settings::instance()->lastDirectory(),
-          tr("All files (*.kar *.mid *.midi *.wrk);;"
+          tr("All files (*.kar *.mid *.midi *.rmi *.wrk);;"
           "Karaoke files (*.kar);;"
           "MIDI Files (*.mid *.midi);;"
+          "RIFF MIDI Files (*.rmi);;"
           "Cakewalk files (*.wrk)") );
     if (!fileName.isEmpty() && isSupported(fileName)) {
         stop();
@@ -459,23 +488,25 @@ void GUIPlayer::setup()
             m_player->setPort(m_midiOut);
             connectOutPort();
         }
-        conn = m_midiOut->currentConnection();
-        Settings::instance()->setAdvanced(m_connections->advanced());
-        Settings::instance()->setLastOutputBackend(m_midiOut->backendName());
-        Settings::instance()->setLastOutputConnection(conn.first);
+        if (m_midiOut != nullptr) {
+            conn = m_midiOut->currentConnection();
+            Settings::instance()->setAdvanced(m_connections->advanced());
+            Settings::instance()->setLastOutputBackend(m_midiOut->backendName());
+            Settings::instance()->setLastOutputConnection(conn.first);
+        }
     }
 }
 
 void GUIPlayer::applySettings()
 {
-    static QPalette defaultPalette = qApp->palette(); //(QColor(0xa0,0xa0,0xa0));
+    static QPalette defaultPalette = qApp->palette(); //style()->standardPalette();
     static QPalette darkPalette(QColor(0x30,0x30,0x30));
 #if defined(Q_OS_WINDOWS)
     m_snapper.SetEnabled(Settings::instance()->winSnap());
 #endif
     qApp->setPalette( Settings::instance()->getDarkMode() ? darkPalette : defaultPalette );
+    qApp->setStyle( Settings::instance()->getStyle() );
 
-    //m_ui->actionPianoPlayer->setIcon(IconUtils::GetIcon\("audio-midi"\));
     m_ui->actionFileInfo->setIcon(IconUtils::GetIcon("dialog-information"));
     m_ui->actionPlayList->setIcon(IconUtils::GetIcon("view-media-playlist"));
     m_ui->actionAbout->setIcon(IconUtils::GetIcon("help-about"));
@@ -497,10 +528,22 @@ void GUIPlayer::applySettings()
     m_ui->actionJump->setIcon(IconUtils::GetIcon("go-jump"));
     m_ui->customizeToolBar->setIcon(IconUtils::GetIcon("settings"));
     m_ui->menuPlaylistRepeat->setIcon(IconUtils::GetIcon("media-playlist-repeat"));
+    m_ui->actionLoop->setIcon(IconUtils::GetIcon("looping"));
+    m_ui->actionContents->setIcon(IconUtils::GetIcon("help-contents"));
+    m_ui->actionWebSite->setIcon(IconUtils::GetIcon("viewhtml"));
+    m_ui->actionLoadSongSettings->setIcon(IconUtils::GetIcon("document-open"));
+    m_ui->actionSaveSongSettings->setIcon(IconUtils::GetIcon("document-save"));
+    m_ui->actionSearch->setIcon(IconUtils::GetIcon("edit-find"));
 
+#if QT_VERSION >= QT_VERSION_CHECK(6,0,0)
+    foreach(QMenu *mnu, m_ui->menuBar->findChildren<QMenu*>()) {
+        mnu->setPalette(qApp->palette());
+    }
+#endif
     m_lyrics->applySettings();
     m_pianola->applySettings();
     m_channels->applySettings();
+    m_helpWindow->applySettings();
 }
 
 void GUIPlayer::preferences()
@@ -516,6 +559,9 @@ void GUIPlayer::playerFinished()
     m_player->resetPosition();
     updateTimeLabel(0);
     updatePositionWidgets();
+    if (Settings::instance()->autoSongSettings()) {
+        slotSaveSongSettings();
+    }
     if (m_repeat == CurrentSong) {
         QTimer::singleShot(0, this, &GUIPlayer::play);
     } else if ( Settings::instance()->autoAdvance()) {
@@ -546,6 +592,9 @@ void GUIPlayer::playerStopped()
         updateState(StoppedState);
     }
     updatePositionWidgets();
+    if (Settings::instance()->autoSongSettings()) {
+        slotSaveSongSettings();
+    }
 }
 
 void GUIPlayer::updateTempoLabel(float ftempo)
@@ -595,6 +644,10 @@ void GUIPlayer::positionSliderMoved(int value)
         m_newSliderPosition = value;
         m_player->beatByTickPosition(m_newSliderPosition);
         m_ui->lblPos->setText(m_player->currentBeatStr());
+        // tooltip
+        QString tip = m_player->currentBeatStr();
+        m_ui->positionSlider->setToolTip(tip);
+        QToolTip::showText(QCursor::pos(), tip, this);
     }
 }
 
@@ -603,6 +656,10 @@ void GUIPlayer::positionSliderReleased()
     m_player->setPosition(m_newSliderPosition);
     m_player->beatByTickPosition(m_newSliderPosition);
     m_ui->lblPos->setText(m_player->currentBeatStr());
+    // tooltip
+    QString tip = m_player->currentBeatStr();
+    m_ui->positionSlider->setToolTip(tip);
+    QToolTip::showText(QCursor::pos(), tip, this);
     if (m_state == PausedState) {
         QTimer::singleShot(0, this, &GUIPlayer::play);
     }
@@ -652,6 +709,25 @@ void GUIPlayer::jump()
     }
 }
 
+void GUIPlayer::loop()
+{
+    auto active = m_ui->actionLoop->isChecked();
+    //qDebug() << Q_FUNC_INFO << active;
+    if (active) {
+        LoopDialog dlg;
+        dlg.setLastBar(m_player->song()->lastBar());
+        dlg.initValues(m_player->loopStart(), m_player->loopEnd());
+        if (dlg.exec() == QDialog::Accepted) {
+            m_player->setLoop(dlg.getFromBar(), dlg.getToBar());
+        } else {
+            m_ui->actionLoop->setChecked(false);
+            m_player->setLoop(false);
+        }
+    } else {
+        m_player->setLoop(false);
+    }
+}
+
 void GUIPlayer::pitchShift(int value)
 {
     m_player->setPitchShift(value);
@@ -702,15 +778,27 @@ void GUIPlayer::dragEnterEvent( QDragEnterEvent * event )
 
 void GUIPlayer::dropEvent( QDropEvent * event )
 {
-    QString data = event->mimeData()->text();
-    QString fileName = QUrl(data).toLocalFile();
-    if ( isSupported(fileName) ) {
-        stop();
-        openFile(fileName);
-        event->accept();
-    } else {
-        QMessageBox::warning(this, QSTR_APPNAME,
-            QString("Dropped file %1 is not supported").arg(fileName));
+    if (event->mimeData()->hasText()) {
+        QStringList list, data;
+#if (QT_VERSION < QT_VERSION_CHECK(5,15,0))
+        data = event->mimeData()->text().split(QChar::LineFeed, QString::SkipEmptyParts);
+#else
+        data = event->mimeData()->text().split(QChar::LineFeed, Qt::SkipEmptyParts);
+#endif
+        foreach(const QString& fileName, data) {
+            QString localFileName = QUrl(fileName).toLocalFile();
+            if ( isSupported(localFileName) ) {
+                list.append(localFileName);
+            } else {
+                QMessageBox::warning(this, QSTR_APPNAME,
+                    tr("Dropped file %1 is not supported").arg(fileName));
+            }
+        }
+        if (!list.isEmpty()) {
+            stop();
+            openFileList(list);
+            event->accept();
+        }
     }
 }
 
@@ -719,6 +807,7 @@ bool GUIPlayer::isSupported(QString fileName)
     return  fileName.endsWith(".mid", Qt::CaseInsensitive) ||
             fileName.endsWith(".midi", Qt::CaseInsensitive) ||
             fileName.endsWith(".kar", Qt::CaseInsensitive) ||
+            fileName.endsWith(".rmi", Qt::CaseInsensitive) ||
             fileName.endsWith(".wrk", Qt::CaseInsensitive);
 }
 
@@ -818,8 +907,24 @@ void GUIPlayer::showEvent(QShowEvent *event)
 {
     QMainWindow::showEvent(event);
     if (m_firstShown) {
-        restoreGeometry(Settings::instance()->mainWindowGeometry());
-        restoreState(Settings::instance()->mainWindowState());
+        const QByteArray geometry = Settings::instance()->mainWindowGeometry();
+        const QByteArray state = Settings::instance()->mainWindowState();
+        if (geometry.isEmpty()) {
+            const QRect availableGeometry =
+#if QT_VERSION < QT_VERSION_CHECK(5,14,0)
+                    QApplication::desktop()->availableGeometry(this);
+#else
+                    screen()->availableGeometry();
+#endif
+            //qDebug() << Q_FUNC_INFO << availableGeometry;
+            setGeometry(QStyle::alignedRect(Qt::LeftToRight, Qt::AlignCenter,
+                                            size(), availableGeometry));
+        } else {
+            restoreGeometry(geometry);
+        }
+        if (!state.isEmpty()) {
+            restoreState(state);
+        }
         QStringList actions = Settings::instance()->toolbarActions();
         if (!actions.empty()) {
             m_ui->toolBar->clear();
@@ -839,13 +944,20 @@ void GUIPlayer::showEvent(QShowEvent *event)
 void GUIPlayer::createLanguageMenu()
 {
     QString currentLang = Settings::instance()->language();
+    if (currentLang.isEmpty()) {
+        QLocale locale;
+        if (locale.language() == QLocale::C || locale.language() == QLocale::English)
+            currentLang = "C";
+        else
+            currentLang = locale.name().left(2);
+    }
     QActionGroup *languageGroup = new QActionGroup(this);
     languageGroup->setExclusive(true);
     connect(languageGroup, &QActionGroup::triggered, this, &GUIPlayer::slotSwitchLanguage);
     QDir dir(Settings::localeDirectory());
-    QStringList fileNames = dir.entryList({"*.qm"}, QDir::NoFilter, QDir::NoSort);
+    QStringList fileNames = dir.entryList({"dmidiplayer*.qm"}, QDir::NoFilter, QDir::NoSort);
     QStringList locales;
-    locales << "en";
+    locales << "C";
     foreach (const QString& fileName, fileNames) {
         QFileInfo f(fileName);
         QString locale = f.fileName();
@@ -859,7 +971,7 @@ void GUIPlayer::createLanguageMenu()
     m_ui->menuLanguage->clear();
     foreach (const QString& loc, locales) {
         QLocale qlocale(loc);
-        QString localeName = loc == "en" ? QLocale::languageToString(qlocale.language()) : qlocale.nativeLanguageName();
+        QString localeName = loc == "C" ? QLocale::languageToString(QLocale::English) : qlocale.nativeLanguageName();
         QAction *action = new QAction(localeName.section(" ", 0, 0), this);
         action->setCheckable(true);
         action->setData(loc);
@@ -884,10 +996,7 @@ void GUIPlayer::slotAboutTranslation()
 
 void GUIPlayer::retranslateUi()
 {
-    m_trq->load("qt_" + Settings::instance()->language(), Settings::systemLocales());
-    m_trp->load("dmidiplayer_" + Settings::instance()->language(), Settings::localeDirectory());
-    m_trl->load("drumstick-widgets_" + Settings::instance()->language(), Settings::drumstickLocales());
-    Settings::instance()->retranslatePalettes();
+    Settings::instance()->loadTranslations();
     m_ui->retranslateUi(this);
     m_connections->retranslateUi();
     m_pianola->retranslateUi();
@@ -898,16 +1007,17 @@ void GUIPlayer::retranslateUi()
     m_recentFiles->retranslateUi();
     m_playList->retranslateUi();
     m_toolbarEditor->retranslateUi();
+    m_helpWindow->retranslateUi();
 }
 
 void GUIPlayer::slotSwitchLanguage(QAction *action)
 {
     QString lang = action->data().toString();
     QLocale qlocale(lang);
-    QString localeName = qlocale.nativeLanguageName();
+    QString langName = lang == "C" ? QLocale::languageToString(QLocale::English) : qlocale.nativeLanguageName();
     if ( QMessageBox::question (this, tr("Language Changed"),
             tr("The language for this application is going to change to %1. "
-               "Do you want to continue?").arg(localeName),
+               "Do you want to continue?").arg(langName),
             QMessageBox::Yes | QMessageBox::No) == QMessageBox::Yes )
     {
         Settings::instance()->setLanguage(lang);
@@ -926,35 +1036,42 @@ void GUIPlayer::slotFileInfo()
     if (m_player->song()->currentFile().isEmpty())
         infostr = tr("<b>No file loaded</b>");
     else {
-        infostr = tr("<b>File:</b><br>%1<br>").arg(m_player->song()->currentFile());
-
-        QString s = m_player->song()->getText(Sequence::Copyright).join(tr("<br>"));
+        infostr = tr("File: <b>%1</b><br>").arg(m_player->song()->currentFile());
+        QString s = m_player->song()->getFileFormat();
         if (!s.isEmpty())
-            infostr += tr("<b>Copyright:</b>%1<br>").arg(s);
+            infostr += tr("Format: <b>%1</b><br>").arg(s);
+
+        s = m_player->song()->getText(Sequence::Copyright).join(tr("<br>"));
+        if (!s.isEmpty())
+            infostr += tr("Copyright: <b>%1</b><br>").arg(s);
 
         s = m_player->song()->getText(Sequence::KarFileType).join(tr("<br>"));
         if (!s.isEmpty())
-            infostr += tr("<b>Karaoke type:</b>%1<br>").arg(s);
+            infostr += tr("Karaoke type: <b>%1</b><br>").arg(s);
 
         s = m_player->song()->getText(Sequence::KarVersion).join(tr("<br>"));
         if (!s.isEmpty())
-            infostr += tr("<b>Karaoke version:</b>%1<br>").arg(s);
+            infostr += tr("Karaoke version: <b>%1</b><br>").arg(s);
 
         s = m_player->song()->getText(Sequence::KarInformation).join(tr("<br>"));
         if (!s.isEmpty())
-            infostr += tr("<b>Karaoke info:</b>%1<br>").arg(s);
+            infostr += tr("Karaoke info: <b>%1</b><br>").arg(s);
 
         s = m_player->song()->getText(Sequence::KarLanguage).join(tr("<br>"));
         if (!s.isEmpty())
-            infostr += tr("<b>Karaoke language:</b>%1<br>").arg(s);
+            infostr += tr("Karaoke language: <b>%1</b><br>").arg(s);
 
         s = m_player->song()->getText(Sequence::KarTitles).join(tr("<br>"));
         if (!s.isEmpty())
-            infostr += tr("<b>Karaoke title:</b>%1<br>").arg(s);
+            infostr += tr("Karaoke title: <b>%1</b><br>").arg(s);
 
         s = m_player->song()->getText(Sequence::KarWhatever).join(tr("<br>"));
         if (!s.isEmpty())
-            infostr += tr("<b>Karaoke things:</b>%1<br>").arg(s);
+            infostr += tr("Karaoke things: <b>%1</b><br>").arg(s);
+
+        s = m_player->song()->getMetadataInfo();
+        if (!s.isEmpty())
+            infostr += s;
     }
     infostr.replace(QChar::LineSeparator, tr("<br>"));
     QMessageBox::information(this, tr("SMF Information"), infostr );
@@ -996,3 +1113,131 @@ void GUIPlayer::slotPlaylistRepeat(QAction *action)
         m_repeat = Nothing;
     }
 }
+
+void GUIPlayer::slotHelp()
+{
+    m_helpWindow->show();
+}
+
+void GUIPlayer::slotOpenWebSite()
+{
+    QUrl url(QStringLiteral("https://dmidiplayer.sourceforge.io"));
+    QDesktopServices::openUrl(url);
+}
+
+void GUIPlayer::slotVolume(int channel, qreal vol)
+{
+    m_player->setVolume(channel, vol);
+}
+
+void GUIPlayer::slotMuted(int channel, bool mute)
+{
+    m_player->setMuted(channel, mute);
+}
+
+void GUIPlayer::slotLocked(int channel, bool lock)
+{
+    m_player->setLocked(channel, lock);
+}
+
+void GUIPlayer::slotPatch(int channel, int patch)
+{
+    m_player->setPatch(channel, patch);
+}
+
+void GUIPlayer::slotSaveSongSettings()
+{
+    QString songName = m_player->song()->currentFile();
+    //qDebug() << Q_FUNC_INFO << songName;
+    if (!songName.isEmpty()) {
+        QDir dataDir(QStandardPaths::writableLocation(QStandardPaths::HomeLocation) + "/.dmidiplayer");
+        QString fileName = QString("%1/%2.cfg").arg(dataDir.absolutePath(), songName);
+        QSettings songSettings(fileName, QSettings::IniFormat);
+        songSettings.beginGroup("Global");
+        songSettings.setValue("file", m_player->song()->currentFullFileName());
+        songSettings.setValue("encoding", QString::fromLatin1(m_player->song()->currentCharset()));
+        songSettings.setValue("volume", m_ui->volumeSlider->value());
+        songSettings.setValue("pitch", m_ui->spinPitch->value());
+        songSettings.setValue("timeskew", m_ui->sliderTempo->value());
+        songSettings.endGroup();
+        for(int i = 0; i < MIDI_STD_CHANNELS; ++i ) {
+            if ( m_player->song()->channelUsed(i) ) {
+                songSettings.beginGroup(QString("MIDI Channel %1").arg(i+1,2));
+                songSettings.setValue("name", m_channels->channelName(i));
+                songSettings.setValue("muted", m_channels->isChannelMuted(i));
+                songSettings.setValue("solo", m_channels->isChannelSoloed(i));
+                songSettings.setValue("locked", m_channels->isChannelLocked(i));
+                songSettings.setValue("patch", m_channels->channelPatch(i));
+                songSettings.setValue("level", m_channels->channelLevel(i));
+                songSettings.endGroup();
+            }
+        }
+        songSettings.sync();
+    }
+}
+
+void GUIPlayer::slotLoadSongSettings()
+{
+    int vol, pitch, skew, pgm;
+    bool locked, muted, solo;
+    QString songName = m_player->song()->currentFile();
+    //qDebug() << Q_FUNC_INFO << songName;
+    if (!songName.isEmpty()) {
+        QDir dataDir(QStandardPaths::writableLocation(QStandardPaths::HomeLocation) + "/.dmidiplayer");
+        QString fileName = QString("%1/%2.cfg").arg(dataDir.absolutePath(), songName);
+        QSettings songSettings(fileName, QSettings::IniFormat);
+        songSettings.beginGroup("Global");
+        //QString songfile = songSettings.value("file").toString();
+        //qDebug() << Q_FUNC_INFO << songfile;
+        QString encoding = songSettings.value("encoding").toString();
+        if (!encoding.isEmpty()) {
+            m_player->song()->setCurrentCharset(encoding.toLatin1());
+            m_lyrics->initSong(m_player->song());
+        }
+        vol = songSettings.value("volume", 100).toInt();
+        m_ui->volumeSlider->setValue(vol);
+        volumeSlider(vol);
+        pitch = songSettings.value("pitch", 0).toInt();
+        m_ui->spinPitch->setValue(pitch);
+        pitchShift(pitch);
+        skew = songSettings.value("timeskew", 100).toInt();
+        m_ui->sliderTempo->setValue(skew);
+        tempoSlider(skew);
+        songSettings.endGroup();
+        for(int i = 0; i < MIDI_STD_CHANNELS; ++i ) {
+            QString grpName = QString("MIDI Channel %1").arg(i+1,2);
+            if ( songSettings.childGroups().contains(grpName) ) {
+                songSettings.beginGroup(grpName);
+                QString name = songSettings.value("name").toString();
+                if (name.isEmpty())
+                    name =  m_player->song()->channelLabel(i);
+                if (!name.isEmpty()) {
+                    m_channels->setChannelName(i, name);
+                    m_pianola->slotLabel(i, name);
+                }
+                muted = songSettings.value("muted", false).toBool();
+                m_channels->setMuteChannel(i, muted);
+                solo = songSettings.value("solo", false).toBool();
+                m_channels->setSoloChannel(i, solo);
+                pgm = songSettings.value("patch", -1).toInt();
+                m_channels->setPatchChannel(i, pgm);
+                locked = songSettings.value("locked", false).toBool();
+                m_channels->setLockChannel(i, locked);
+                vol = songSettings.value("level", 100).toInt();
+                m_channels->setLevelChannel(i, vol);
+                songSettings.endGroup();
+            }
+        }
+    }
+}
+
+void GUIPlayer::slotSearch()
+{
+    QString site = QLatin1String("http://www.vanbasco.com/midisearch.html");
+    QString lang = Settings::instance()->language();
+    if (lang == QLatin1String("es") || lang == QLatin1String("it")) {
+        site = QLatin1String("http://www.vanbasco.com/") + lang + QLatin1String("/midisearch.html");
+    }
+    //qDebug() << Q_FUNC_INFO << site;
+    QDesktopServices::openUrl(QUrl(site));
+}
