@@ -1,6 +1,6 @@
 ﻿/*
     Drumstick MIDI File Player Multiplatform Program
-    Copyright (C) 2006-2021, Pedro Lopez-Cabanillas <plcl@users.sf.net>
+    Copyright (C) 2006-2022, Pedro Lopez-Cabanillas <plcl@users.sf.net>
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -24,7 +24,6 @@
 #include <chrono>
 #include <typeinfo>
 
-//#include <QDebug>
 #include <QAbstractEventDispatcher>
 #include <QThread>
 #include <QtMath>
@@ -46,9 +45,39 @@ void SequencePlayer::initChannels()
     }
 }
 
+bool SequencePlayer::isLoopEnabled() const
+{
+    return m_loopEnabled;
+}
+
+void SequencePlayer::setLoop(bool enabled)
+{
+    m_loopEnabled = enabled;
+}
+
+void SequencePlayer::setLoop(int loopStart, int loopEnd)
+{
+    m_loopEnabled = true;
+    m_loopStart = loopStart;
+    m_loopEnd = loopEnd;
+}
+
+int SequencePlayer::loopEnd() const
+{
+    return m_loopEnd;
+}
+
+int SequencePlayer::loopStart() const
+{
+    return m_loopStart;
+}
+
 SequencePlayer::SequencePlayer() :
     m_port(nullptr),
     m_songPosition(0),
+    m_loopEnabled(false),
+    m_loopStart(0),
+    m_loopEnd(0),
     m_echoResolution(50),
     m_pitchShift(0),
     m_volumeFactor(100),
@@ -202,19 +231,19 @@ void SequencePlayer::playEvent(MIDIEvent* ev)
             TempoEvent* event = static_cast<TempoEvent*>(ev);
             auto tempo = event->tempo();
             m_song.updateTempo(tempo);
-            //qDebug() << m_songPosition << ev->tick() << " Tempo: " << tempo << "bpm:" << bpm(tempo); // << "ticks2millis:" << m_song.ticks2millis();
+            //qDebug() << m_songPosition << ev->tick() << " Tempo: " << tempo << "bpm:" << event->bpm();
             emit tempoChanged(tempo);
         } else
         if (typeid(*ev) == beatId) {
-            //qDebug() << m_songPosition << ev->tick() << " Meta-event";
             BeatEvent* event = static_cast<BeatEvent*>(ev);
             emit beat(event->bar(), event->beat(), event->barLength());
             m_latestBeat = event;
-            //qDebug() << m_songPosition << "bar:beat ->" << event->bar() << ":" << event->beat();
+            //qDebug() << m_songPosition << ev->tick() << "bar:beat ->" << event->bar() << ":" << event->beat();
         } else
         if (typeid(*ev) == timeSigId) {
             TimeSignatureEvent* event = static_cast<TimeSignatureEvent*>(ev);
             emit timeSignature(event->tag(), event->numerator(), event->denominator());
+            //qDebug() << m_songPosition << ev->tick() << "TimeSignature:" << event->tag() << event->numerator() << event->denominator();
         } else
         if (typeid(*ev) == keySigId) {
             KeySignatureEvent* event = static_cast<KeySignatureEvent*>(ev);
@@ -231,6 +260,8 @@ void SequencePlayer::playerLoop()
 {
     using namespace std::chrono;
     typedef system_clock Clock;
+    static const std::type_info& beatId = typeid(BeatEvent);
+    int currentBar{ 0 };
     long echoPosition{ 0 }, echoTicks{ 0 };
     milliseconds deltaTime{0}, echoDelta{ m_echoResolution };
     Clock::time_point currentTime{ Clock::now() },
@@ -244,32 +275,44 @@ void SequencePlayer::playerLoop()
     timeBeginPeriod(1);
 #endif
 
-    while (m_song.hasMoreEvents() && !thread()->isInterruptionRequested()) {
-        MIDIEvent* ev = m_song.nextEvent();
-        if (ev->delta() > 0) {
-            deltaTime = m_song.deltaTimeOfEvent(ev);
-            echoDelta = m_song.timeOfTicks(m_echoResolution);
-            nextTime = currentTime + deltaTime;
-            nextEcho = currentTime + echoDelta;
-            echoPosition = m_songPosition;
-            while (nextEcho < nextTime) {
-                dispatcher->processEvents(eventFilter);
-                std::this_thread::sleep_until(nextEcho);
-                echoPosition += echoDelta.count();
-                echoTicks += m_echoResolution;
-                emit songEchoTime(echoPosition, echoTicks);
-                currentTime = Clock::now();
-                nextEcho = currentTime + echoDelta;
+    do {
+        while (m_song.hasMoreEvents() && !thread()->isInterruptionRequested()) {
+            MIDIEvent* ev = m_song.nextEvent();
+            if (typeid(*ev) == beatId) {
+                currentBar = static_cast<BeatEvent*>(ev)->bar();
+                if (isLoopEnabled() && (currentBar > loopEnd())) {
+                    break;
+                }
             }
-            dispatcher->processEvents(eventFilter);
-            std::this_thread::sleep_until(nextTime);
-            echoTicks = ev->tick();
-            m_songPosition += deltaTime.count();
-            currentTime = Clock::now();
-            emit songEchoTime(m_songPosition, ev->tick());
+            if (ev->delta() > 0) {
+                deltaTime = m_song.deltaTimeOfEvent(ev);
+                echoDelta = m_song.timeOfTicks(m_echoResolution);
+                nextTime = currentTime + deltaTime;
+                nextEcho = currentTime + echoDelta;
+                echoPosition = m_songPosition;
+                while (nextEcho < nextTime) {
+                    dispatcher->processEvents(eventFilter);
+                    std::this_thread::sleep_until(nextEcho);
+                    echoPosition += echoDelta.count();
+                    echoTicks += m_echoResolution;
+                    emit songEchoTime(echoPosition, echoTicks);
+                    currentTime = Clock::now();
+                    nextEcho = currentTime + echoDelta;
+                }
+                dispatcher->processEvents(eventFilter);
+                std::this_thread::sleep_until(nextTime);
+                echoTicks = ev->tick();
+                m_songPosition += deltaTime.count();
+                currentTime = Clock::now();
+                emit songEchoTime(m_songPosition, ev->tick());
+                //qDebug() << "echo:" << m_songPosition << ev->tick() << deltaTime.count();
+            }
+            playEvent(ev);
         }
-        playEvent(ev);
-    }
+        if (isLoopEnabled()) {
+            jumpToBar(loopStart());
+        }
+    } while (m_song.hasMoreEvents() && !thread()->isInterruptionRequested());
 
 #ifdef WIN32
     timeEndPeriod(1);
@@ -337,6 +380,9 @@ void SequencePlayer::loadFile(QString fileName)
     m_songPosition = 0;
     m_firstBeat = m_song.firstBeat();
     m_latestBeat = m_firstBeat;
+    m_loopStart = 1;
+    m_loopEnd = m_song.lastBar();
+    m_loopEnabled = false;
 }
 
 void SequencePlayer::pause()
@@ -358,6 +404,7 @@ void SequencePlayer::resetPosition()
 void SequencePlayer::setPosition(long pos)
 {
     //qDebug() << Q_FUNC_INFO << pos;
+    allNotesOff();
     m_song.setTickPosition(pos);
     m_songPosition = pos;
 }
@@ -373,7 +420,7 @@ void SequencePlayer::setVolumeFactor(unsigned int vol)
 {
     //qDebug() << Q_FUNC_INFO;
     m_volumeFactor = vol;
-    if (m_port != 0) {
+    if (m_port != nullptr) {
         for(int chan = 0; chan < MIDI_STD_CHANNELS; ++chan) {
             int value = m_volume[chan];
             value = qFloor(value * m_volumeFactor / 100.0);
@@ -387,7 +434,7 @@ void SequencePlayer::setVolumeFactor(unsigned int vol)
 void SequencePlayer::allNotesOff()
 {
     //qDebug() << Q_FUNC_INFO;
-    if (m_port != 0) {
+    if (m_port != nullptr) {
         for(int chan = 0; chan < MIDI_STD_CHANNELS; ++chan) {
             m_port->sendController(chan, ControllerEvent::MIDI_CTL_ALL_NOTES_OFF, 0);
             m_port->sendController(chan, ControllerEvent::MIDI_CTL_ALL_SOUNDS_OFF, 0);
@@ -398,7 +445,7 @@ void SequencePlayer::allNotesOff()
 void SequencePlayer::resetControllers()
 {
     //qDebug() << Q_FUNC_INFO;
-    if (m_port != 0) {
+    if (m_port != nullptr) {
         for(int chan = 0; chan < MIDI_STD_CHANNELS; ++chan) {
             m_port->sendController(chan, ControllerEvent::MIDI_CTL_RESET_CONTROLLERS, 0);
             m_port->sendController(chan, ControllerEvent::MIDI_CTL_MSB_MAIN_VOLUME, 100);
@@ -406,6 +453,17 @@ void SequencePlayer::resetControllers()
     }
 }
 
+void SequencePlayer::resetPrograms()
+{
+    if (m_port != nullptr) {
+        for(int chan = 0; chan < MIDI_STD_CHANNELS; ++chan) {
+            int pgm = m_locked[chan] ? m_lockedpgm[chan] : 0;
+            m_port->sendProgram(chan, pgm);
+            //qDebug() << Q_FUNC_INFO << chan << m_locked[chan] << pgm;
+        }
+    }
+}
+
 void SequencePlayer::sendResetMessage()
 {
     static const QByteArray gmreset = QByteArray::fromHex("f07e7f0901f7");
@@ -424,13 +482,14 @@ void SequencePlayer::sendResetMessage()
             m_port->sendSysex(xgreset);
             break;
         }
+        //qDebug() << Q_FUNC_INFO << reset_msg;
     }
 }
 
 void SequencePlayer::sendVolumeEvents()
 {
     //qDebug() << Q_FUNC_INFO;
-    if (m_port != 0) {
+    if (m_port != nullptr) {
         for(int chan = 0; chan < MIDI_STD_CHANNELS; ++chan) {
             int value = m_volume[chan] = 100;
             value = floor(value * m_volumeFactor / 100.0);
@@ -474,8 +533,7 @@ void SequencePlayer::beatForward()
 {
     BeatEvent* ev = m_song.nextBar(m_latestBeat);
     if (ev != nullptr) {
-        setPosition(ev->tick());
-        m_latestBeat = ev;
+        setBeatPosition(ev);
     }
 }
 
@@ -483,17 +541,22 @@ void SequencePlayer::beatBackward()
 {
     BeatEvent* ev = m_song.previousBar(m_latestBeat);
     if (ev != nullptr) {
-        setPosition(ev->tick());
-        m_latestBeat = ev;
+        setBeatPosition(ev);
     }
 }
 
+void SequencePlayer::setBeatPosition(BeatEvent* ev)
+{
+    setPosition(ev->tick());
+    emit beat(ev->bar(), ev->beat(), ev->barLength());
+    m_latestBeat = ev;
+}
+
 void SequencePlayer::jumpToBar(int bar)
 {
     BeatEvent* ev = m_song.jumpToBar(bar);
     if (ev != nullptr) {
-        setPosition(ev->tick());
-        m_latestBeat = ev;
+        setBeatPosition(ev);
     }
 }
 
