@@ -1,6 +1,6 @@
 /*
     Drumstick MIDI File Player Multiplatform Program
-    Copyright (C) 2006-2021, Pedro Lopez-Cabanillas <plcl@users.sf.net>
+    Copyright (C) 2006-2022, Pedro Lopez-Cabanillas <plcl@users.sf.net>
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -16,8 +16,8 @@
     along with this program. If not, see <http://www.gnu.org/licenses/>.
 */
 
-//#include <QDebug>
 #include <QtMath>
+#include <QDebug>
 #include <QFileInfo>
 #include <QRegularExpression>
 #include "sequence.h"
@@ -26,6 +26,7 @@ using namespace drumstick::File;
 using namespace drumstick::rt;
 
 Sequence::Sequence(QObject *parent) : QObject(parent),
+    m_rmid(nullptr),
     m_smf(nullptr),
     m_wrk(nullptr),
     m_uchardetErrors(0),
@@ -46,8 +47,13 @@ Sequence::Sequence(QObject *parent) : QObject(parent),
     m_duration(0),
     m_lastBeat(0),
     m_beatLength(0),
-    m_tick(0)
+    m_tick(0),
+    m_codec(nullptr)
 {
+    m_rmid = new Rmidi(this);
+    connect(m_rmid, &Rmidi::signalRiffData, this, &Sequence::dataHandler);
+    connect(m_rmid, &Rmidi::signalRiffInfo, this, &Sequence::infoHandler);
+
     m_smf = new QSmf(this);
     connect(m_smf, &QSmf::signalSMFHeader, this, &Sequence::smfHeaderEvent);
     connect(m_smf, &QSmf::signalSMFNoteOn, this, &Sequence::smfNoteOnEvent);
@@ -59,7 +65,7 @@ Sequence::Sequence(QObject *parent) : QObject(parent),
     connect(m_smf, &QSmf::signalSMFChanPress, this, &Sequence::smfChanPressEvent);
     connect(m_smf, &QSmf::signalSMFMetaMisc, this, &Sequence::smfMetaEvent);
     connect(m_smf, &QSmf::signalSMFSysex, this, &Sequence::smfSysexEvent);
-    connect(m_smf, &QSmf::signalSMFText, this, &Sequence::smfUpdateLoadProgress);
+    connect(m_smf, &QSmf::signalSMFText2, this, &Sequence::smfUpdateLoadProgress);
     connect(m_smf, &QSmf::signalSMFTempo, this, &Sequence::smfTempoEvent);
     connect(m_smf, &QSmf::signalSMFTrackStart, this, &Sequence::smfTrackStartEvent);
     connect(m_smf, &QSmf::signalSMFTrackEnd, this, &Sequence::smfTrackEnd);
@@ -99,6 +105,7 @@ Sequence::Sequence(QObject *parent) : QObject(parent),
     connect(m_wrk, &QWrk::signalWRKSegment2, this, &Sequence::wrkSegment);
     connect(m_wrk, &QWrk::signalWRKChord, this, &Sequence::wrkChord);
     connect(m_wrk, &QWrk::signalWRKExpression2, this, &Sequence::wrkExpression);
+    connect(m_wrk, &QWrk::signalWRKMarker2, this, &Sequence::wrkMarker);
     /* m_wrk->setTextCodec(nullptr); // important !!! */
 
     m_handle = uchardet_new();
@@ -260,6 +267,7 @@ void Sequence::sort()
     foreach(MIDIEvent* ev, m_list) {
         ev->setDelta(ev->tick() - lastEventTicks);
         lastEventTicks = ev->tick();
+        //qDebug() << typeid (*ev).name() << ev->tick() << ev->delta();
     }
 }
 
@@ -280,6 +288,8 @@ void Sequence::clear()
     m_lowestMidiNote = 127;
     m_highestMidiNote = 0;
     m_curTrack = 0;
+    m_codec = nullptr;
+    m_infoMap.clear();
     for(int i=0; i<MIDI_STD_CHANNELS; ++i) {
         m_channelUsed[i] = false;
         m_channelEvents[i] = 0;
@@ -295,6 +305,8 @@ void Sequence::clear()
     m_typScore.clear();
     m_trkName.clear();
     m_trkChannel.clear();
+    m_currentFile.clear();
+    m_currentFileFull.clear();
     while (!m_list.isEmpty()) {
         delete m_list.takeFirst();
     }
@@ -322,6 +334,8 @@ void Sequence::loadFile(const QString& fileName)
             QString ext = finfo.suffix().toLower();
             if (ext == "wrk") {
                 m_wrk->readFromFile(fileName);
+            } else if (ext == "rmi") {
+                m_rmid->readFromFile(fileName);
             } else if (ext == "mid" || ext == "midi" || ext == "kar") {
                 m_smf->readFromFile(fileName);
             }
@@ -331,9 +345,9 @@ void Sequence::loadFile(const QString& fileName)
                 sort();
                 m_lblName = finfo.fileName();
                 m_currentFile = finfo.fileName();
+                m_currentFileFull = finfo.absoluteFilePath();
             }
             m_charset = QByteArray(uchardet_get_charset(m_handle));
-            //qDebug() << Q_FUNC_INFO << "Detected charset:" << m_charset;
             findCodec();
         } catch (...) {
             qWarning() << "corrupted file";
@@ -347,21 +361,34 @@ int Sequence::numUchardetErrors()
     return m_uchardetErrors;
 }
 
-int Sequence::detectedUchardetMIB() const
+int Sequence::currentMIB() const
 {
     return m_mib;
 }
 
-QByteArray Sequence::detectedCharset() const
+QByteArray Sequence::currentCharset() const
 {
+    //qDebug() << Q_FUNC_INFO << "Current charset:" << m_charset;
     return m_charset;
 }
 
+void Sequence::setCurrentCharset(const QByteArray &charset)
+{
+    m_charset = charset;
+    //qDebug() << Q_FUNC_INFO << "Current charset:" << m_charset;
+    findCodec();
+}
+
 QString Sequence::currentFile() const
 {
     return m_currentFile;
 }
 
+QString Sequence::currentFullFileName() const
+{
+    return m_currentFileFull;
+}
+
 int Sequence::getNumTracks() const
 {
     return m_numTracks;
@@ -378,6 +405,35 @@ void Sequence::appendStringToList(QStringList &list, QString &s, TextType type)
     list.append(s);
 }
 
+QTextCodec *Sequence::codec() const
+{
+    return m_codec;
+}
+
+void Sequence::setCodec(QTextCodec *newCodec)
+{
+    m_codec = newCodec;
+    m_charset = m_codec->name();
+}
+
+QString Sequence::getFileFormat() const
+{
+    return m_fileFormat;
+}
+
+QString Sequence::getMetadataInfo() const
+{
+    if (!m_infoMap.empty()) {
+        QString metadata;
+        QMap<QString,QString>::const_iterator i;
+        for(i = m_infoMap.constBegin(); i != m_infoMap.constEnd(); ++i) {
+            metadata += i.key() + ": <b>" + i.value() + "</b><br/>";
+        }
+        return metadata;
+    }
+    return QString();
+}
+
 QStringList Sequence::getText(const TextType type)
 {
     QStringList output;
@@ -556,10 +612,12 @@ void Sequence::updateTempo(qreal newTempo)
 void Sequence::insertBeats(qint64 ticks)
 {
     if ((ticks > m_tick) && (m_beatLength > 0)) {
+        //qDebug() << Q_FUNC_INFO << ticks << m_tick << m_beatLength;
         qint64 diff = ticks - m_lastBeat;
         while (diff >= m_beatLength) {
             MIDIEvent* ev = new BeatEvent(m_barCount, m_beatCount, m_beatMax);
             ev->setTick(m_lastBeat);
+            //qDebug() << "Beat:" << m_lastBeat << m_barCount << m_beatCount;
             m_list.append(ev);
 
             m_lastBeat += m_beatLength;
@@ -578,6 +636,50 @@ void Sequence::insertBeats(qint64 ticks)
  * SMF (Standard MIDI file) format handling
  * **************************************** */
 
+void Sequence::dataHandler(const QString& dataType, const QByteArray& data)
+{
+    if (dataType == "RMID") {
+        QDataStream ds(data);
+        m_smf->readFromStream(&ds);
+        m_fileFormat += tr(" in RIFF container of type %1").arg(dataType);
+    }
+}
+
+void Sequence::infoHandler(const QString &infoType, const QByteArray &data)
+{
+    const QMap<QString,QString> keyMap{
+        {"IALB", "Album"},
+        {"IARL", "Archival Location"},
+        {"IART", "Artist"},
+        {"ICMS", "Commissioned"},
+        {"ICMT", "Comments"},
+        {"ICOP", "Copyright"},
+        {"ICRD", "Creation date"},
+        {"IENG", "Engineer"},
+        {"IGNR", "Genre"},
+        {"IKEY", "Keywords"},
+        {"IMED", "Medium"},
+        {"INAM", "Name"},
+        {"IPRD", "Product"},
+        {"ISBJ", "Subject"},
+        {"ISFT", "Software"},
+        {"ISRC", "Source"},
+        {"ISR", "Source Form"},
+        {"ITCH", "Technician"}};
+    QString key, value;
+    if (keyMap.contains(infoType)) {
+        key = keyMap[infoType];
+    } else {
+        key = infoType;
+    }
+    if (m_codec == nullptr) {
+        value = QString::fromLatin1(data);
+    } else {
+        value = m_codec->toUnicode(data);
+    }
+    m_infoMap[key] = value;
+}
+
 void Sequence::smfUpdateLoadProgress()
 {
     emit loadingProgress(m_smf->getFilePos());
@@ -599,10 +701,10 @@ void Sequence::appendSMFEvent(MIDIEvent *ev)
 
 void Sequence::smfHeaderEvent(int format, int ntrks, int division)
 {
-    //qDebug() << "SMF Header:" << QString("Format=%1, Tracks=%2, Division=%3").arg(format).arg(ntrks).arg(division);
     m_format = format;
     m_numTracks = ntrks;
     m_division = division;
+    m_fileFormat = tr("SMF type %1").arg(format);
     m_beatLength = m_division;
     m_beatMax = 4;
     m_lastBeat = 0;
@@ -666,6 +768,7 @@ void Sequence::smfProgramEvent(int chan, int patch)
     m_channelEvents[chan]++;
     MIDIEvent* ev = new ProgramChangeEvent (chan, patch);
     appendSMFEvent(ev);
+    //qDebug() << Q_FUNC_INFO << chan << patch;
 }
 
 void Sequence::smfChanPressEvent(int chan, int press)
@@ -847,10 +950,9 @@ void Sequence::appendWRKEvent(long ticks, MIDIEvent* ev)
     if (ev->tag() <= 0) {
         ev->setTag(m_curTrack);
     }
+    //qDebug() << Q_FUNC_INFO << ticks << ev->tag() << typeid(*ev).name();
     m_list.append(ev);
-    if (ticks > m_ticksDuration) {
-        m_ticksDuration = ticks;
-    }
+    m_ticksDuration = qMax((int)ticks, m_ticksDuration);
     insertBeats(ticks);
     wrkUpdateLoadProgress();
 }
@@ -871,7 +973,8 @@ void Sequence::wrkFileHeader(int verh, int verl)
     m_lastBeat = 0;
     m_beatCount = 1;
     m_barCount = 1;
-    m_fileFormat = QString("%1.%2").arg(verh).arg(verl);
+    m_fileFormat = tr("WRK file version v%1.%2").arg(verh).arg(verl);
+    timeCalculations();
     wrkUpdateLoadProgress();
 }
 
@@ -924,34 +1027,28 @@ void Sequence::wrkTrackHeader( const QByteArray& name1,
 
 void Sequence::wrkNoteEvent(int track, long time, int chan, int pitch, int vol, int dur)
 {
-    int channel = chan;
     TrackMapRec rec = m_trackMap[track+1];
-    int key = pitch + rec.pitch;
-    int velocity = vol + rec.velocity;
-    if (rec.channel > -1)
-        channel = rec.channel;
-    if (pitch > m_highestMidiNote)
-        m_highestMidiNote = pitch;
-    if (pitch < m_lowestMidiNote)
-        m_lowestMidiNote = pitch;
+    int key = qBound(0, pitch + rec.pitch, 127);
+    int velocity = qBound(0, vol + rec.velocity, 127);
+    int channel = (rec.channel > -1) ? rec.channel : chan;
+    //qDebug() << Q_FUNC_INFO << "time:" << time << "chan:" << channel << key << velocity << dur;
+    m_highestMidiNote = qMax(pitch, m_highestMidiNote);
+    m_lowestMidiNote = qMin(pitch, m_lowestMidiNote);
     m_channelUsed[channel] = true;
     MIDIEvent* ev = new NoteOnEvent(channel, key, velocity);
     ev->setTag(track+1);
     appendWRKEvent(time, ev);
-    m_channelEvents[channel]++;
     ev = new NoteOffEvent(channel, key, velocity);
     ev->setTag(track+1);
     appendWRKEvent(time + dur, ev);
-    m_channelEvents[channel]++;
+    m_channelEvents[channel]+=2;
 }
 
 void Sequence::wrkKeyPressEvent(int track, long time, int chan, int pitch, int press)
 {
-    int channel = chan;
     TrackMapRec rec = m_trackMap[track+1];
     int key = pitch + rec.pitch;
-    if (rec.channel > -1)
-        channel = rec.channel;
+    int channel = (rec.channel > -1) ? rec.channel : chan;
     m_channelUsed[channel] = true;
     m_channelEvents[channel]++;
     MIDIEvent* ev = new KeyPressEvent(channel, key, press);
@@ -961,10 +1058,8 @@ void Sequence::wrkKeyPressEvent(int track, long time, int chan, int pitch, int p
 
 void Sequence::wrkCtlChangeEvent(int track, long time, int chan, int ctl, int value)
 {
-    int channel = chan;
     TrackMapRec rec = m_trackMap[track+1];
-    if (rec.channel > -1)
-        channel = rec.channel;
+    int channel = (rec.channel > -1) ? rec.channel : chan;
     m_channelUsed[channel] = true;
     m_channelEvents[channel]++;
     MIDIEvent* ev = new ControllerEvent(channel, ctl, value);
@@ -974,10 +1069,8 @@ void Sequence::wrkCtlChangeEvent(int track, long time, int chan, int ctl, int va
 
 void Sequence::wrkPitchBendEvent(int track, long time, int chan, int value)
 {
-    int channel = chan;
     TrackMapRec rec = m_trackMap[track+1];
-    if (rec.channel > -1)
-        channel = rec.channel;
+    int channel = (rec.channel > -1) ? rec.channel : chan;
     m_channelUsed[channel] = true;
     m_channelEvents[channel]++;
     MIDIEvent* ev = new PitchBendEvent(channel, value);
@@ -987,23 +1080,22 @@ void Sequence::wrkPitchBendEvent(int track, long time, int chan, int value)
 
 void Sequence::wrkProgramEvent(int track, long time, int chan, int patch)
 {
-    int channel = chan;
-    TrackMapRec rec = m_trackMap[track+1];
-    if (rec.channel > -1)
-        channel = rec.channel;
-    m_channelUsed[channel] = true;
-    m_channelEvents[channel]++;
-    MIDIEvent* ev = new ProgramChangeEvent(channel, patch);
-    ev->setTag(track+1);
-    appendWRKEvent(time, ev);
+    if (patch >= 0 && patch < 128) {
+        TrackMapRec rec = m_trackMap[track+1];
+        int channel = (rec.channel > -1) ? rec.channel : chan;
+        m_channelUsed[channel] = true;
+        m_channelEvents[channel]++;
+        MIDIEvent* ev = new ProgramChangeEvent(channel, patch);
+        ev->setTag(track+1);
+        appendWRKEvent(time, ev);
+        //qDebug() << Q_FUNC_INFO << track << time << chan << patch;
+    }
 }
 
 void Sequence::wrkChanPressEvent(int track, long time, int chan, int press)
 {
-    int channel = chan;
     TrackMapRec rec = m_trackMap[track+1];
-    if (rec.channel > -1)
-        channel = rec.channel;
+    int channel = (rec.channel > -1) ? rec.channel : chan;
     m_channelUsed[channel] = true;
     m_channelEvents[channel]++;
     MIDIEvent* ev = new ChanPressEvent(channel, press);
@@ -1013,26 +1105,26 @@ void Sequence::wrkChanPressEvent(int track, long time, int chan, int press)
 
 void Sequence::wrkSysexEvent(int track, long time, int bank)
 {
-    SysexEventRec rec;
-    rec.track = track+1;
-    rec.time = time;
-    rec.bank = bank;
-    m_savedSysexEvents.append(rec);
+    if (m_savedSysexEvents.contains(bank)) {
+        SysExEvent *ev = m_savedSysexEvents[bank].clone();
+        ev->setTag(track+1);
+        appendWRKEvent(time, ev);
+    }
     wrkUpdateLoadProgress();
 }
 
 void Sequence::wrkSysexEventBank(int bank, const QString& /*name*/,
         bool autosend, int /*port*/, const QByteArray& data)
 {
-    SysExEvent* ev = new SysExEvent(data);
-    if (autosend)
-        appendWRKEvent(0, ev->clone());
-    foreach(const SysexEventRec& rec, m_savedSysexEvents) {
-        if (rec.bank == bank) {
-            appendWRKEvent(rec.time, ev->clone());
-        }
+    SysExEvent ev(data);
+    if (autosend) {
+        auto savedTrack = m_curTrack;
+        m_curTrack = 0;
+        appendWRKEvent(0, ev.clone());
+        m_curTrack = savedTrack;
+    } else {
+        m_savedSysexEvents[bank] = ev;
     }
-    delete ev;
     wrkUpdateLoadProgress();
 }
 
@@ -1109,20 +1201,20 @@ void Sequence::wrkVariableRecord(const QString &name, const QByteArray &data)
 void Sequence::wrkTempoEvent(long time, int tempo)
 {
     double bpm = tempo / 100.0;
-    MIDIEvent* ev = new TempoEvent(qRound ( 6e7 / bpm ) );
+    TempoEvent* ev = new TempoEvent(6e7 / bpm);
     appendWRKEvent(time, ev);
+    //qDebug() << Q_FUNC_INFO << time << bpm;
     if (time == 0) {
-        updateTempo(tempo);
+        updateTempo(ev->tempo());
     }
 }
 
 void Sequence::wrkTrackPatch(int track, int patch)
 {
-    int channel = 0;
     TrackMapRec rec = m_trackMap[track+1];
-    if (rec.channel > -1)
-        channel = rec.channel;
+    int channel = (rec.channel > -1) ? rec.channel : 0;
     wrkProgramEvent(track+1, 0, channel, patch);
+    //qDebug() << Q_FUNC_INFO << track+1 << patch;
 }
 
 void Sequence::wrkNewTrackHeader( const QByteArray& data,
@@ -1159,29 +1251,25 @@ void Sequence::wrkTrackName(int trackno, const QByteArray &data)
 
 void Sequence::wrkTrackVol(int track, int vol)
 {
-    int channel = 0;
     int lsb, msb;
     TrackMapRec rec = m_trackMap[track+1];
-    if (rec.channel > -1)
-        channel = rec.channel;
+    int channel = (rec.channel > -1) ? rec.channel : 0;
     if (vol < 128)
-        wrkCtlChangeEvent(track+1, 0, channel, ControllerEvent::MIDI_CTL_MSB_MAIN_VOLUME, vol);
+        wrkCtlChangeEvent(track, 0, channel, ControllerEvent::MIDI_CTL_MSB_MAIN_VOLUME, vol);
     else {
         lsb = vol % 0x80;
         msb = vol / 0x80;
-        wrkCtlChangeEvent(track+1, 0, channel, ControllerEvent::MIDI_CTL_LSB_MAIN_VOLUME, lsb);
-        wrkCtlChangeEvent(track+1, 0, channel, ControllerEvent::MIDI_CTL_MSB_MAIN_VOLUME, msb);
+        wrkCtlChangeEvent(track, 0, channel, ControllerEvent::MIDI_CTL_LSB_MAIN_VOLUME, lsb);
+        wrkCtlChangeEvent(track, 0, channel, ControllerEvent::MIDI_CTL_MSB_MAIN_VOLUME, msb);
     }
 }
 
 void Sequence::wrkTrackBank(int track, int bank)
 {
     // assume GM/GS bank method
-    int channel = 0;
     int lsb, msb;
     TrackMapRec rec = m_trackMap[track+1];
-    if (rec.channel > -1)
-        channel = rec.channel;
+    int channel = (rec.channel > -1) ? rec.channel : 0;
     lsb = bank % 0x80;
     msb = bank / 0x80;
     wrkCtlChangeEvent(track+1, 0, channel, ControllerEvent::MIDI_CTL_MSB_BANK, msb);
@@ -1208,10 +1296,7 @@ void Sequence::wrkExpression(int track, long time, int /*code*/, const QByteArra
 
 void Sequence::wrkTimeSignatureEvent(int bar, int num, int den)
 {
-    MIDIEvent* ev = new TimeSignatureEvent(num, den);
-    m_beatMax = num;
-    m_beatLength = m_division * 4 / den;
-
+    TimeSignatureEvent* ev = new TimeSignatureEvent(num, den);
     TimeSigRec newts;
     newts.bar = bar;
     newts.num = num;
@@ -1231,12 +1316,15 @@ void Sequence::wrkTimeSignatureEvent(int bar, int num, int den)
         if (!found) {
             TimeSigRec& lasts = m_bars.last();
             newts.time = lasts.time +
-                    (lasts.num * 4 / lasts.den * m_division * (bar - lasts.bar));
+                    (lasts.num * 4 * m_division / lasts.den * (bar - lasts.bar));
             m_bars.append(newts);
         }
     }
     ev->setTag( bar );
+    //qDebug() << Q_FUNC_INFO << ev->tag() << ev->numerator() << ev->denominator();
     appendWRKEvent(newts.time, ev);
+    m_beatMax = num;
+    m_beatLength = m_division * 4 / den;
 }
 
 void Sequence::wrkKeySig(int bar, int alt)
@@ -1252,6 +1340,15 @@ void Sequence::wrkKeySig(int bar, int alt)
     appendWRKEvent(time, ev);
 }
 
+void Sequence::wrkMarker(long time, int smpte, const QByteArray &data)
+{
+    Q_UNUSED(smpte)
+    //qDebug() << Q_FUNC_INFO << time << smpte << data;
+    if (!data.isEmpty()) {
+        appendWRKmetadata(1, time, TextType::Marker, data);
+    }
+}
+
 void Sequence::wrkEndOfFile()
 {
     wrkUpdateLoadProgress();
@@ -1269,7 +1366,7 @@ QString Sequence::channelLabel(int channel)
     if ((channel >= 0) && (channel < MIDI_STD_CHANNELS) &&
         (!m_channelLabel[channel].isEmpty())) {
         if (m_codec == nullptr)
-            return QString::fromLocal8Bit(m_channelLabel[channel]);
+            return QString::fromLatin1(m_channelLabel[channel]);
         else
             return m_codec->toUnicode(m_channelLabel[channel]);
     }
