@@ -1,6 +1,6 @@
 ﻿/*
     Drumstick MIDI File Player Multiplatform Program
-    Copyright (C) 2006-2021, Pedro Lopez-Cabanillas <plcl@users.sf.net>
+    Copyright (C) 2006-2022, Pedro Lopez-Cabanillas <plcl@users.sf.net>
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -32,7 +32,10 @@
 using namespace drumstick::rt;
 using namespace drumstick::widgets;
 
-Settings::Settings(QObject *parent) : QObject(parent)
+Settings::Settings(QObject *parent) : QObject(parent),
+    m_trq(nullptr),
+    m_trp(nullptr),
+    m_trl(nullptr)
 {
     ResetDefaults();
     ReadSettings();
@@ -111,7 +114,11 @@ QString Settings::systemLocales()
 #if defined(TRANSLATIONS_EMBEDDED)
     return QLatin1String(":/");
 #else
-    return QLibraryInfo::location(QLibraryInfo::TranslationsPath);
+    #if QT_VERSION < QT_VERSION_CHECK(6,0,0)
+        return QLibraryInfo::location(QLibraryInfo::TranslationsPath);
+    #else
+        return QLibraryInfo::path(QLibraryInfo::TranslationsPath);
+    #endif
 #endif
 }
 
@@ -135,6 +142,9 @@ void Settings::ReadSettings()
 {
     SettingsFactory settings;
     internalRead(*settings.getQSettings());
+#if defined (Q_OS_MACOS)
+    forceSettings(*settings.getQSettings());
+#endif
 }
 
 void Settings::SaveSettings()
@@ -157,9 +167,23 @@ void Settings::SaveToFile(const QString &filepath)
 
 void Settings::internalRead(QSettings &settings)
 {
+    QFont defaultFont = QGuiApplication::font();
+
     settings.beginGroup("MainWindow");
-    m_mainWindowGeometry = settings.value("Geometry").toByteArray();
-    m_mainWindowState = settings.value("State").toByteArray();
+    m_mainWindowGeometry = settings.value("Geometry", QByteArray()).toByteArray();
+    m_mainWindowState = settings.value("State", QByteArray()).toByteArray();
+    settings.endGroup();
+
+    settings.beginGroup("ChannelsWindow");
+    m_channelsWindowGeometry = settings.value("Geometry", QByteArray()).toByteArray();
+    m_channelsWindowState = settings.value("State", QByteArray()).toByteArray();
+    m_soloVolumeReduction = settings.value("SoloReduction", 50).toInt();
+    settings.endGroup();
+
+    settings.beginGroup("HelpWindow");
+    m_helpWindowGeometry = settings.value("Geometry", QByteArray()).toByteArray();
+    m_helpWindowState = settings.value("State", QByteArray()).toByteArray();
+    m_helpWindowFontSize = settings.value("FontSize",  defaultFont.pointSize()).toInt();
     settings.endGroup();
 
     settings.beginGroup(BackendManager::QSTR_DRUMSTICKRT_GROUP);
@@ -186,10 +210,12 @@ void Settings::internalRead(QSettings &settings)
     m_showStatusBar = settings.value("ShowStatusBar", true).toBool();
     m_showToolBar = settings.value("ShowToolBar", true).toBool();
     m_drumsChannel = settings.value("DrumsChannel", MIDI_GM_STD_DRUM_CHANNEL+1).toInt();
-    m_language = settings.value("Language", QLocale::system().name()).toString();
+    m_language = settings.value("Language").toString();
     m_darkMode = settings.value("DarkMode", false).toBool();
     m_autoPlay = settings.value("AutoPlay", true).toBool();
     m_autoAdvance = settings.value("AutoAdvance", true).toBool();
+    m_autoSongSettings = settings.value("AutoSongSettings", true).toBool();
+    m_octaveSubscript = settings.value("OctaveSubscript", false).toBool();
     QDir defDir(dataDirectory());
     m_lastPlayList = settings.value("LastPlaylist", defDir.absoluteFilePath("examples.lst")).toString();
     bool internalIcons = false;
@@ -210,23 +236,33 @@ void Settings::internalRead(QSettings &settings)
     settings.endGroup();
 
     settings.beginGroup("TextSettings");
-    QFont lyricsFont;
-    if (lyricsFont.fromString(settings.value("LyricsFont", "Sans Serif,36").toString())) {
-        m_lyricsFont = lyricsFont;
+    QFont tempFont, defTextFont = QGuiApplication::font();
+    defTextFont.setPointSize(36);
+    if (tempFont.fromString(settings.value("LyricsFont", defTextFont.toString()).toString())) {
+        m_lyricsFont = tempFont;
+    } else {
+        m_lyricsFont = defTextFont;
     }
     m_futureColor = QColor(settings.value("FutureColor", qApp->palette().color(QPalette::WindowText).name(QColor::HexRgb)).toString());
     m_pastColor = QColor(settings.value("PastColor", QColor(Qt::gray).name(QColor::HexRgb)).toString());
+    m_lyricsWindowGeometry = settings.value("Geometry", QByteArray()).toByteArray();
+    m_lyricsWindowState = settings.value("State", QByteArray()).toByteArray();
     settings.endGroup();
 
     settings.beginGroup("PlayerPianoSettings");
-    QFont notesFont;
-    if (notesFont.fromString(settings.value("NotesFont", "Sans Serif,50").toString())) {
-        m_notesFont = notesFont;
+    QFont defNoteFont = QGuiApplication::font();
+    defNoteFont.setPointSize(50);
+    if (tempFont.fromString(settings.value("NotesFont", defNoteFont.toString()).toString())) {
+        m_notesFont = tempFont;
+    } else {
+        m_notesFont = defNoteFont;
     }
     m_highlightPaletteId = settings.value("PaletteId", PAL_SINGLE).toInt();
     m_velocityColor = settings.value("VelocityColor", true).toBool();
     setNamesVisibility(static_cast<LabelVisibility>(settings.value("NamesVisibility", ShowNever).toInt()));
     m_singleColor = QColor(settings.value("SingleColor", m_paletteList[0].getColor(0).name(QColor::HexRgb)).toString());
+    m_pianoWindowGeometry = settings.value("Geometry", QByteArray()).toByteArray();
+    m_pianoWindowState = settings.value("State", QByteArray()).toByteArray();
     settings.endGroup();
 
     emit ValuesChanged();
@@ -261,6 +297,8 @@ void Settings::internalSave(QSettings &settings)
     settings.setValue("DarkMode", m_darkMode);
     settings.setValue("AutoPlay", m_autoPlay);
     settings.setValue("AutoAdvance", m_autoAdvance);
+    settings.setValue("AutoSongSettings", m_autoSongSettings);
+    settings.setValue("OctaveSubscript", m_octaveSubscript);
     settings.setValue("LastPlaylist", m_lastPlayList);
 #if defined(Q_OS_WINDOWS)
     settings.setValue("WindowSnapping", m_winSnap);
@@ -276,6 +314,8 @@ void Settings::internalSave(QSettings &settings)
     settings.setValue("LyricsFont", m_lyricsFont.toString());
     settings.setValue("FutureColor", m_futureColor.name(QColor::HexRgb));
     settings.setValue("PastColor", m_pastColor.name(QColor::HexRgb));
+    settings.setValue("Geometry", m_lyricsWindowGeometry);
+    settings.setValue("State", m_lyricsWindowState);
     settings.endGroup();
 
     settings.beginGroup("PlayerPianoSettings");
@@ -284,11 +324,208 @@ void Settings::internalSave(QSettings &settings)
     settings.setValue("PaletteId", m_highlightPaletteId);
     settings.setValue("NamesVisibility", m_namesVisibility);
     settings.setValue("SingleColor", m_singleColor.name(QColor::HexRgb));
+    settings.setValue("Geometry", m_pianoWindowGeometry);
+    settings.setValue("State", m_pianoWindowState);
+    settings.endGroup();
+
+    settings.beginGroup("HelpWindow");
+    settings.setValue("Geometry", m_helpWindowGeometry);
+    settings.setValue("State", m_helpWindowState);
+    settings.setValue("FontSize", m_helpWindowFontSize);
+    settings.endGroup();
+
+    settings.beginGroup("ChannelsWindow");
+    settings.setValue("Geometry", m_channelsWindowGeometry);
+    settings.setValue("State", m_channelsWindowState);
+    settings.setValue("SoloReduction", m_soloVolumeReduction);
     settings.endGroup();
 
     settings.sync();
 }
 
+void Settings::forceSettings(QSettings &settings)
+{
+#if defined(Q_OS_MACOS)
+    settings.beginGroup("DLS Synth");
+    settings.setValue("reverb_dls", true);
+    settings.endGroup();
+#endif
+}
+
+bool Settings::octaveSubscript() const
+{
+    return m_octaveSubscript;
+}
+
+void Settings::setOctaveSubscript(bool newOctaveSubscript)
+{
+    m_octaveSubscript = newOctaveSubscript;
+}
+
+bool Settings::autoSongSettings() const
+{
+    return m_autoSongSettings;
+}
+
+void Settings::setAutoSongSettings(bool newAutoSongSettings)
+{
+    m_autoSongSettings = newAutoSongSettings;
+}
+
+int Settings::soloVolumeReduction() const
+{
+    return m_soloVolumeReduction;
+}
+
+void Settings::setSoloVolumeReduction(int newSoloVolumeReduction)
+{
+    m_soloVolumeReduction = newSoloVolumeReduction;
+}
+
+int Settings::helpWindowFontSize() const
+{
+    return m_helpWindowFontSize;
+}
+
+void Settings::setHelpWindowFontSize(int newHelpWindowFontSize)
+{
+    m_helpWindowFontSize = newHelpWindowFontSize;
+}
+
+void Settings::loadTranslations()
+{
+    QLocale loc;
+    QString lang = language();
+    if (!lang.isEmpty()) {
+        loc = QLocale(lang);
+    }
+    if (m_trq) {
+        QCoreApplication::removeTranslator(m_trq);
+        delete m_trq;
+    }
+    if (m_trp) {
+        QCoreApplication::removeTranslator(m_trp);
+        delete m_trp;
+    }
+    if (m_trl) {
+        QCoreApplication::removeTranslator(m_trl);
+        delete m_trl;
+    }
+    if (loc.language() != QLocale::C && loc.language() != QLocale::English) {
+        bool ok = false;
+        m_trq = new QTranslator(this);
+        if (m_trq->load(loc, "qt", "_", Settings::systemLocales())) {
+            ok = QCoreApplication::installTranslator(m_trq);
+        }
+        if(!ok) {
+            qWarning() << "Failure loading Qt system translations for" << lang
+                       << "from" << Settings::systemLocales();
+            delete m_trq;
+        }
+        ok = false;
+        m_trp = new QTranslator(this);
+        if (m_trp->load(loc, "dmidiplayer", "_", Settings::localeDirectory())) {
+            ok = QCoreApplication::installTranslator(m_trp);
+        }
+        if (!ok) {
+            qWarning() << "Failure loading application translations for" << lang
+                       << "from" << Settings::localeDirectory();
+            delete m_trp;
+        }
+        ok = false;
+        m_trl = new QTranslator(this);
+        if (m_trl->load(loc, "drumstick-widgets", "_", Settings::drumstickLocales())) {
+            ok = QCoreApplication::installTranslator(m_trl);
+        }
+        if (!ok) {
+            qWarning() << "Failure loading widgets library translations for" << lang
+                       << "from" << Settings::drumstickLocales();
+            delete m_trl;
+        }
+    }
+    retranslatePalettes();
+}
+
+const QByteArray &Settings::pianoWindowState() const
+{
+    return m_pianoWindowState;
+}
+
+void Settings::setPianoWindowState(const QByteArray &newPianoWindowState)
+{
+    m_pianoWindowState = newPianoWindowState;
+}
+
+const QByteArray &Settings::pianoWindowGeometry() const
+{
+    return m_pianoWindowGeometry;
+}
+
+void Settings::setPianoWindowGeometry(const QByteArray &newPianoWindowGeometry)
+{
+    m_pianoWindowGeometry = newPianoWindowGeometry;
+}
+
+const QByteArray &Settings::lyricsWindowState() const
+{
+    return m_lyricsWindowState;
+}
+
+void Settings::setLyricsWindowState(const QByteArray &newLyricsWindowState)
+{
+    m_lyricsWindowState = newLyricsWindowState;
+}
+
+const QByteArray &Settings::lyricsWindowGeometry() const
+{
+    return m_lyricsWindowGeometry;
+}
+
+void Settings::setLyricsWindowGeometry(const QByteArray &newLyricsWindowGeometry)
+{
+    m_lyricsWindowGeometry = newLyricsWindowGeometry;
+}
+
+const QByteArray &Settings::helpWindowState() const
+{
+    return m_helpWindowState;
+}
+
+void Settings::setHelpWindowState(const QByteArray &newHelpWindowState)
+{
+    m_helpWindowState = newHelpWindowState;
+}
+
+const QByteArray &Settings::helpWindowGeometry() const
+{
+    return m_helpWindowGeometry;
+}
+
+void Settings::setHelpWindowGeometry(const QByteArray &newHelpWindowGeometry)
+{
+    m_helpWindowGeometry = newHelpWindowGeometry;
+}
+
+const QByteArray &Settings::channelsWindowState() const
+{
+    return m_channelsWindowState;
+}
+
+void Settings::setChannelsWindowState(const QByteArray &newChannelsWindowState)
+{
+    m_channelsWindowState = newChannelsWindowState;
+}
+
+const QByteArray &Settings::channelsWindowGeometry() const
+{
+    return m_channelsWindowGeometry;
+}
+
+void Settings::setChannelsWindowGeometry(const QByteArray &newChannelsWindowGeometry)
+{
+    m_channelsWindowGeometry = newChannelsWindowGeometry;
+}
+
 int Settings::toolbarButtonStyle() const
 {
     return m_toolbarButtonStyle;
@@ -451,6 +688,10 @@ void Settings::setVelocityColor(bool velocityColor)
 
 QString Settings::language() const
 {
+    //qDebug() << Q_FUNC_INFO << m_language;
+    if (m_language.length() > 2) {
+        return m_language.left(2);
+    }
     return m_language;
 }
 
