@@ -30,7 +30,7 @@ p, li { white-space: pre-wrap; }
 &lt;p style=&quot; margin-top:20px; margin-bottom:20px; margin-left:20px; margin-right:20px; -qt-block-indent:0; text-indent:0px;&quot;&gt;&lt;img src=&quot;:/dmidiplayer128.png&quot; style=&quot;float: left;&quot; /&gt;&lt;span style=&quot; font-family:&apos;Sans Serif&apos;; font-size:16pt; font-weight:600;&quot;&gt;%PROJECT_DESCRIPTION%&lt;/span&gt;&lt;/p&gt;
 &lt;p style=&quot; margin-top:12px; margin-bottom:12px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;&quot;&gt;&lt;span style=&quot; font-family:&apos;Sans Serif&apos;; font-size:16pt; font-weight:600;&quot;&gt;Version %VERSION%&lt;/span&gt;&lt;/p&gt;
 &lt;p style=&quot; margin-top:12px; margin-bottom:12px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;&quot;&gt;&lt;span style=&quot; font-family:&apos;Sans Serif&apos;;&quot;&gt;Using &lt;/span&gt;&lt;a href=&quot;https://drumstick.sourceforge.io&quot;&gt;&lt;span style=&quot; font-family:&apos;DejaVu Sans&apos;; text-decoration: underline; color:#0057ae;&quot;&gt;Drumstick MIDI C++ libraries &lt;/span&gt;&lt;/a&gt;&lt;span style=&quot; font-family:&apos;Sans Serif&apos;;&quot;&gt;v%DRUMSTICK_VERSION%&lt;/span&gt;&lt;/p&gt;
-&lt;p style=&quot; margin-top:12px; margin-bottom:12px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;&quot;&gt;&lt;span style=&quot; font-family:&apos;Sans Serif&apos;;&quot;&gt;Copyright © 2006-2021, Pedro Lopez-Cabanillas &amp;lt;&lt;/span&gt;&lt;a href=&quot;mailto:plcl@users.sf.net?subject=dmidiplayer&quot;&gt;&lt;span style=&quot; text-decoration: underline; color:#007af4;&quot;&gt;plcl@users.sf.net&lt;/span&gt;&lt;/a&gt;&lt;span style=&quot; font-family:&apos;Sans Serif&apos;;&quot;&gt;&amp;gt;&lt;/span&gt;&lt;/p&gt;
+&lt;p style=&quot; margin-top:12px; margin-bottom:12px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;&quot;&gt;&lt;span style=&quot; font-family:&apos;Sans Serif&apos;;&quot;&gt;Copyright © 2006-2022, Pedro Lopez-Cabanillas &amp;lt;&lt;/span&gt;&lt;a href=&quot;mailto:plcl@users.sf.net?subject=dmidiplayer&quot;&gt;&lt;span style=&quot; text-decoration: underline; color:#007af4;&quot;&gt;plcl@users.sf.net&lt;/span&gt;&lt;/a&gt;&lt;span style=&quot; font-family:&apos;Sans Serif&apos;;&quot;&gt;&amp;gt;&lt;/span&gt;&lt;/p&gt;
 &lt;p style=&quot; margin-top:12px; margin-bottom:12px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;&quot;&gt;&lt;span style=&quot; font-family:&apos;Sans Serif&apos;;&quot;&gt;This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation; either version 3 of the License, or (at your option) any later version.&lt;/span&gt;&lt;/p&gt;
 &lt;p style=&quot; margin-top:12px; margin-bottom:12px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;&quot;&gt;&lt;span style=&quot; font-family:&apos;Sans Serif&apos;;&quot;&gt;This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.&lt;/span&gt;&lt;/p&gt;
 &lt;p style=&quot; margin-top:12px; margin-bottom:12px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;&quot;&gt;&lt;span style=&quot; font-family:&apos;Sans Serif&apos;;&quot;&gt;You should have received a copy of the GNU General Public License along with this program; if not, &lt;/span&gt;&lt;span style=&quot; font-family:&apos;Hack&apos;;&quot;&gt;see &amp;lt;&lt;/span&gt;&lt;a href=&quot;https://www.gnu.org/licenses/&quot;&gt;&lt;span style=&quot; text-decoration: underline; color:#007af4;&quot;&gt;https://www.gnu.org/licenses/&lt;/span&gt;&lt;/a&gt;&lt;span style=&quot; font-family:&apos;Hack&apos;;&quot;&gt;&amp;gt;.&lt;/span&gt;&lt;/p&gt;&lt;/body&gt;&lt;/html&gt;</source>
@@ -41,7 +41,7 @@ p, li { white-space: pre-wrap; }
 &lt;p style=&quot; margin-top:20px; margin-bottom:20px; margin-left:20px; margin-right:20px; -qt-block-indent:0; text-indent:0px;&quot;&gt;&lt;img src=&quot;:/dmidiplayer128.png&quot; style=&quot;float: left;&quot; /&gt;&lt;span style=&quot; font-family:&apos;Sans Serif&apos;; font-size:16pt; font-weight:600;&quot;&gt;%PROJECT_DESCRIPTION%&lt;/span&gt;&lt;/p&gt;
 &lt;p style=&quot; margin-top:12px; margin-bottom:12px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;&quot;&gt;&lt;span style=&quot; font-family:&apos;Sans Serif&apos;; font-size:16pt; font-weight:600;&quot;&gt;Versión %VERSION%&lt;/span&gt;&lt;/p&gt;
 &lt;p style=&quot; margin-top:12px; margin-bottom:12px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;&quot;&gt;&lt;span style=&quot; font-family:&apos;Sans Serif&apos;;&quot;&gt;Usando &lt;/span&gt;&lt;a href=&quot;https://drumstick.sourceforge.io&quot;&gt;&lt;span style=&quot; font-family:&apos;DejaVu Sans&apos;; text-decoration: underline; color:#0057ae;&quot;&gt;Bibliotecas Drumstick MIDI C++ &lt;/span&gt;&lt;/a&gt;&lt;span style=&quot; font-family:&apos;Sans Serif&apos;;&quot;&gt;v%DRUMSTICK_VERSION%&lt;/span&gt;&lt;/p&gt;
-&lt;p style=&quot; margin-top:12px; margin-bottom:12px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;&quot;&gt;&lt;span style=&quot; font-family:&apos;Sans Serif&apos;;&quot;&gt;Copyright © 2006-2021, Pedro Lopez-Cabanillas &amp;lt;&lt;/span&gt;&lt;a href=&quot;mailto:plcl@users.sf.net?subject=dmidiplayer&quot;&gt;&lt;span style=&quot; text-decoration: underline; color:#007af4;&quot;&gt;plcl@users.sf.net&lt;/span&gt;&lt;/a&gt;&lt;span style=&quot; font-family:&apos;Sans Serif&apos;;&quot;&gt;&amp;gt;&lt;/span&gt;&lt;/p&gt;
+&lt;p style=&quot; margin-top:12px; margin-bottom:12px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;&quot;&gt;&lt;span style=&quot; font-family:&apos;Sans Serif&apos;;&quot;&gt;Copyright © 2006-2022, Pedro Lopez-Cabanillas &amp;lt;&lt;/span&gt;&lt;a href=&quot;mailto:plcl@users.sf.net?subject=dmidiplayer&quot;&gt;&lt;span style=&quot; text-decoration: underline; color:#007af4;&quot;&gt;plcl@users.sf.net&lt;/span&gt;&lt;/a&gt;&lt;span style=&quot; font-family:&apos;Sans Serif&apos;;&quot;&gt;&amp;gt;&lt;/span&gt;&lt;/p&gt;
 &lt;p style=&quot; margin-top:12px; margin-bottom:12px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;&quot;&gt;&lt;span style=&quot; font-family:&apos;Sans Serif&apos;;&quot;&gt;Este programa es software libre: puede redistribuirlo y/o modificarlo bajo los términos de la Licencia General Pública de GNU publicada por la Free Software Foundation, ya sea la versión 3 de la Licencia, o (a su elección) cualquier versión posterior.&lt;/span&gt;&lt;/p&gt;
 &lt;p style=&quot; margin-top:12px; margin-bottom:12px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;&quot;&gt;&lt;span style=&quot; font-family:&apos;Sans Serif&apos;;&quot;&gt;Este programa se distribuye con la esperanza de que sea útil pero SIN NINGUNA GARANTÍA; incluso sin la garantía implícita de MERCANTIBILIDAD o CALIFICADA PARA UN PROPÓSITO EN PARTICULAR. Vea la Licencia General Pública de GNU para más detalles.&lt;/span&gt;&lt;/p&gt;
 &lt;p style=&quot; margin-top:12px; margin-bottom:12px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;&quot;&gt;&lt;span style=&quot; font-family:&apos;Sans Serif&apos;;&quot;&gt;Usted ha debido de recibir una copia de la Licencia General Pública de GNU junto con este programa. Si no, &lt;/span&gt;&lt;span style=&quot; font-family:&apos;Hack&apos;;&quot;&gt;vea &amp;lt;&lt;/span&gt;&lt;a href=&quot;https://www.gnu.org/licenses/&quot;&gt;&lt;span style=&quot; text-decoration: underline; color:#007af4;&quot;&gt;https://www.gnu.org/licenses/&lt;/span&gt;&lt;/a&gt;&lt;span style=&quot; font-family:&apos;Hack&apos;;&quot;&gt;&amp;gt;.&lt;/span&gt;&lt;/p&gt;&lt;/body&gt;&lt;/html&gt;</translation>
@@ -77,6 +77,22 @@ p, li { white-space: pre-wrap; }
         <source>Patch (sound setting)</source>
         <translation>Programa (sonido)</translation>
     </message>
+    <message>
+        <source>Show all channels</source>
+        <translation>Mostrar todos los canales</translation>
+    </message>
+    <message>
+        <source>Hide all channels</source>
+        <translation>Ocultar todos los canales</translation>
+    </message>
+    <message>
+        <source>View Full Screen</source>
+        <translation>Ver en pantalla completa</translation>
+    </message>
+    <message>
+        <source>Channel %1</source>
+        <translation>Canal %1</translation>
+    </message>
 </context>
 <context>
     <name>Connections</name>
@@ -128,7 +144,7 @@ p, li { white-space: pre-wrap; }
     </message>
     <message>
         <source>&lt;b&gt;File:&lt;/b&gt;&lt;br&gt;%1&lt;br&gt;</source>
-        <translation>&lt;b&gt;Archivo:&lt;/b&gt;&lt;br&gt;%1&lt;br&gt;</translation>
+        <translation type="vanished">&lt;b&gt;Archivo:&lt;/b&gt;&lt;br&gt;%1&lt;br&gt;</translation>
     </message>
     <message>
         <source>&lt;br&gt;</source>
@@ -136,31 +152,31 @@ p, li { white-space: pre-wrap; }
     </message>
     <message>
         <source>&lt;b&gt;Copyright:&lt;/b&gt;%1&lt;br&gt;</source>
-        <translation>&lt;b&gt;Copyright:&lt;/b&gt;%1&lt;br&gt;</translation>
+        <translation type="vanished">&lt;b&gt;Copyright:&lt;/b&gt;%1&lt;br&gt;</translation>
     </message>
     <message>
         <source>&lt;b&gt;Karaoke type:&lt;/b&gt;%1&lt;br&gt;</source>
-        <translation>&lt;b&gt;Tipo de Karaoke:&lt;/b&gt;%1&lt;br&gt;</translation>
+        <translation type="vanished">&lt;b&gt;Tipo de Karaoke:&lt;/b&gt;%1&lt;br&gt;</translation>
     </message>
     <message>
         <source>&lt;b&gt;Karaoke version:&lt;/b&gt;%1&lt;br&gt;</source>
-        <translation>&lt;b&gt;Versión de Karaoke :&lt;/b&gt;%1&lt;br&gt;</translation>
+        <translation type="vanished">&lt;b&gt;Versión de Karaoke :&lt;/b&gt;%1&lt;br&gt;</translation>
     </message>
     <message>
         <source>&lt;b&gt;Karaoke info:&lt;/b&gt;%1&lt;br&gt;</source>
-        <translation>&lt;b&gt;Información de Karaoke:&lt;/b&gt;%1&lt;br&gt;</translation>
+        <translation type="vanished">&lt;b&gt;Información de Karaoke:&lt;/b&gt;%1&lt;br&gt;</translation>
     </message>
     <message>
         <source>&lt;b&gt;Karaoke language:&lt;/b&gt;%1&lt;br&gt;</source>
-        <translation>&lt;b&gt;Idioma del Karaoke:&lt;/b&gt;%1&lt;br&gt;</translation>
+        <translation type="vanished">&lt;b&gt;Idioma del Karaoke:&lt;/b&gt;%1&lt;br&gt;</translation>
     </message>
     <message>
         <source>&lt;b&gt;Karaoke title:&lt;/b&gt;%1&lt;br&gt;</source>
-        <translation>&lt;b&gt;Título del Karaoke:&lt;/b&gt;%1&lt;br&gt;</translation>
+        <translation type="vanished">&lt;b&gt;Título del Karaoke:&lt;/b&gt;%1&lt;br&gt;</translation>
     </message>
     <message>
         <source>&lt;b&gt;Karaoke things:&lt;/b&gt;%1&lt;br&gt;</source>
-        <translation>&lt;b&gt;Cosas del Karaoke:&lt;/b&gt;%1&lt;br&gt;</translation>
+        <translation type="vanished">&lt;b&gt;Cosas del Karaoke:&lt;/b&gt;%1&lt;br&gt;</translation>
     </message>
     <message>
         <source>SMF Information</source>
@@ -172,7 +188,7 @@ p, li { white-space: pre-wrap; }
     </message>
     <message>
         <source>All files (*.kar *.mid *.midi *.wrk);;Karaoke files (*.kar);;MIDI Files (*.mid *.midi);;Cakewalk files (*.wrk)</source>
-        <translation>Todos los archivos (*.kar *.mid *.midi *.wrk);;Archivos de Karaoke (*.kar);;Archivos MIDI (*.mid *.midi);;Archivos de Cakewalk (*.wrk)</translation>
+        <translation type="vanished">Todos los archivos (*.kar *.mid *.midi *.wrk);;Archivos de Karaoke (*.kar);;Archivos MIDI (*.mid *.midi);;Archivos de Cakewalk (*.wrk)</translation>
     </message>
     <message>
         <source>Playlist not saved</source>
@@ -190,6 +206,79 @@ p, li { white-space: pre-wrap; }
         <source>Bar number:</source>
         <translation>Número de compás:</translation>
     </message>
+    <message>
+        <source>Please, load a song</source>
+        <translation>Por favor, cargue una canción</translation>
+    </message>
+    <message>
+        <source>Playing</source>
+        <translation>Reproduciendo</translation>
+    </message>
+    <message>
+        <source>Paused</source>
+        <translation>En pausa</translation>
+    </message>
+    <message>
+        <source>Stopped</source>
+        <translation>Parado</translation>
+    </message>
+    <message>
+        <source>Not initialized</source>
+        <translation>Sin inicializar</translation>
+    </message>
+    <message>
+        <source>Loading %1 file...</source>
+        <translation>Cargando archivo %1...</translation>
+    </message>
+    <message>
+        <source>file %1 couldn&apos;t be opened</source>
+        <translation>archivo %1 no ha podido abrirse</translation>
+    </message>
+    <message>
+        <source>Dropped file %1 is not supported</source>
+        <translation>Archivo %1 soltado no está soportado</translation>
+    </message>
+    <message>
+        <source>All files (*.kar *.mid *.midi *.rmi *.wrk);;Karaoke files (*.kar);;MIDI Files (*.mid *.midi);;RIFF MIDI Files (*.rmi);;Cakewalk files (*.wrk)</source>
+        <translation>Todos los archivos (*.kar *.mid *.midi *.rmi *.wrk);;Archivos de Karaoke (*.kar);;Archivos MIDI (*.mid *.midi);;Archivos MIDI RIFF (*.rmi);;Archivos de Cakewalk (*.wrk)</translation>
+    </message>
+    <message>
+        <source>File: &lt;b&gt;%1&lt;/b&gt;&lt;br&gt;</source>
+        <translation>Archivo: &lt;b&gt;%1&lt;/b&gt;&lt;br&gt;</translation>
+    </message>
+    <message>
+        <source>Format: &lt;b&gt;%1&lt;/b&gt;&lt;br&gt;</source>
+        <translation>Formato: &lt;b&gt;%1&lt;/b&gt;&lt;br&gt;</translation>
+    </message>
+    <message>
+        <source>Copyright: &lt;b&gt;%1&lt;/b&gt;&lt;br&gt;</source>
+        <translation>Copyright: &lt;b&gt;%1&lt;/b&gt;&lt;br&gt;</translation>
+    </message>
+    <message>
+        <source>Karaoke type: &lt;b&gt;%1&lt;/b&gt;&lt;br&gt;</source>
+        <translation>Tipo de karaoke: &lt;b&gt;%1&lt;/b&gt;&lt;br&gt;</translation>
+    </message>
+    <message>
+        <source>Karaoke version: &lt;b&gt;%1&lt;/b&gt;&lt;br&gt;</source>
+        <translation>Versi
+Versión de karaoke: &lt;b&gt;%1&lt;/b&gt;&lt;br&gt;</translation>
+    </message>
+    <message>
+        <source>Karaoke info: &lt;b&gt;%1&lt;/b&gt;&lt;br&gt;</source>
+        <translation>Información de karaoke: &lt;b&gt;%1&lt;/b&gt;&lt;br&gt;</translation>
+    </message>
+    <message>
+        <source>Karaoke language: &lt;b&gt;%1&lt;/b&gt;&lt;br&gt;</source>
+        <translation>Idiooma de karaoke: &lt;b&gt;%1&lt;/b&gt;&lt;br&gt;</translation>
+    </message>
+    <message>
+        <source>Karaoke title: &lt;b&gt;%1&lt;/b&gt;&lt;br&gt;</source>
+        <translation>Título de karaoke: &lt;b&gt;%1&lt;/b&gt;&lt;br&gt;</translation>
+    </message>
+    <message>
+        <source>Karaoke things: &lt;b&gt;%1&lt;/b&gt;&lt;br&gt;</source>
+        <translation>Varios: &lt;b&gt;%1&lt;/b&gt;&lt;br&gt;</translation>
+    </message>
 </context>
 <context>
     <name>GUIPlayerClass</name>
@@ -505,6 +594,73 @@ p, li { white-space: pre-wrap; }
         <source>Whole Playlist</source>
         <translation>Toda la lista de reproducción</translation>
     </message>
+    <message>
+        <source>Loop</source>
+        <translation>Bucle</translation>
+    </message>
+    <message>
+        <source>Loop between two bars</source>
+        <translation>Bucle entre dos compases</translation>
+    </message>
+    <message>
+        <source>Contents</source>
+        <translation>Contenido</translation>
+    </message>
+    <message>
+        <source>Web Site</source>
+        <translation>Sitio web</translation>
+    </message>
+    <message>
+        <source>Song Settings</source>
+        <translation>Ajustes de canción</translation>
+    </message>
+    <message>
+        <source>Load</source>
+        <translation>Cargar</translation>
+    </message>
+    <message>
+        <source>Load the stored settings for the current song</source>
+        <translation>Cargar los ajustes almacenados para la canción actual</translation>
+    </message>
+    <message>
+        <source>Save</source>
+        <translation>Guardar</translation>
+    </message>
+    <message>
+        <source>Save the settings of the current song</source>
+        <translation>Guardar los ajustes para la canción actual</translation>
+    </message>
+    <message>
+        <source>Search MIDI files</source>
+        <translation>Buscar archivos MIDI</translation>
+    </message>
+</context>
+<context>
+    <name>HelpWindow</name>
+    <message>
+        <source>&amp;Home</source>
+        <translation>&amp;Inicio</translation>
+    </message>
+    <message>
+        <source>&amp;Back</source>
+        <translation>&amp;Atrás</translation>
+    </message>
+    <message>
+        <source>Close</source>
+        <translation>Cerrar</translation>
+    </message>
+    <message>
+        <source>Zoom In</source>
+        <translation>Ampliar</translation>
+    </message>
+    <message>
+        <source>Zoom Out</source>
+        <translation>Reducir</translation>
+    </message>
+    <message>
+        <source>Help: %1</source>
+        <translation type="vanished">Ayuda: %1</translation>
+    </message>
 </context>
 <context>
     <name>InstrumentSet</name>
@@ -1025,15 +1181,38 @@ p, li { white-space: pre-wrap; }
         <translation>Percusión %1</translation>
     </message>
 </context>
+<context>
+    <name>LoopDialog</name>
+    <message>
+        <source>Loop Start and End bars</source>
+        <translation>Compases de inicio y fin</translation>
+    </message>
+    <message>
+        <source>Loop Start:</source>
+        <translation>Compás inicial:</translation>
+    </message>
+    <message>
+        <source>Loop End:</source>
+        <translation>Compás final:</translation>
+    </message>
+    <message>
+        <source>Input Error</source>
+        <translation>Error de entrada</translation>
+    </message>
+    <message>
+        <source>%1 is not greater than %2</source>
+        <translation>%1 no es mayor que %2</translation>
+    </message>
+</context>
 <context>
     <name>Lyrics</name>
     <message>
         <source>Lyrics Viewer</source>
-        <translation>Visor de letras</translation>
+        <translation type="vanished">Visor de letras</translation>
     </message>
     <message>
         <source>Lyrics</source>
-        <translation>Letras</translation>
+        <translation type="vanished">Letras</translation>
     </message>
     <message>
         <source>Track:</source>
@@ -1097,7 +1276,7 @@ p, li { white-space: pre-wrap; }
     </message>
     <message>
         <source>Lyrics Viewer (%1)</source>
-        <translation>Visor de letra (%1)</translation>
+        <translation type="vanished">Visor de letra (%1)</translation>
     </message>
     <message>
         <source>Font</source>
@@ -1136,7 +1315,7 @@ p, li { white-space: pre-wrap; }
     </message>
     <message>
         <source>MIDI Channels</source>
-        <translation>Canales MIDI</translation>
+        <translation type="vanished">Canales MIDI</translation>
     </message>
     <message>
         <source>Show all channels</source>
@@ -1226,7 +1405,7 @@ p, li { white-space: pre-wrap; }
     <name>PrefsDialog</name>
     <message>
         <source>Dialog</source>
-        <translation>Diálogo</translation>
+        <translation type="vanished">Diálogo</translation>
     </message>
     <message>
         <source>General</source>
@@ -1238,7 +1417,7 @@ p, li { white-space: pre-wrap; }
     </message>
     <message>
         <source>Sticky Window Snapping</source>
-        <translation>Acoplamiento de ventanas pegajosas</translation>
+        <translation type="vanished">Acoplamiento de ventanas pegajosas</translation>
     </message>
     <message>
         <source>Forced Dark Mode</source>
@@ -1344,18 +1523,38 @@ p, li { white-space: pre-wrap; }
         <source>Advance Automatically to the Next Playlist item</source>
         <translation>Avance automático en la lista de reproducción</translation>
     </message>
+    <message>
+        <source>Preferences</source>
+        <translation>Preferencias</translation>
+    </message>
+    <message>
+        <source>Solo Button % Volume Reduction:</source>
+        <translation>Reducción de volumen % del botón Solo:</translation>
+    </message>
+    <message>
+        <source>Sticky Window Borders</source>
+        <translation>Bordes de ventana pegajosos</translation>
+    </message>
+    <message>
+        <source>Automatically Load and Save Song Settings</source>
+        <translation>Cargar y guardar automáticamente los ajustes de canción</translation>
+    </message>
+    <message>
+        <source>Octave Subscript Designation</source>
+        <translation>Designación de octava como subíndice</translation>
+    </message>
 </context>
 <context>
     <name>QCoreApplication</name>
     <message>
-        <source>Copyright (C) 2006-2021 Pedro Lopez-Cabanillas
+        <source>Copyright (C) 2006-2022 Pedro Lopez-Cabanillas
 This program comes with ABSOLUTELY NO WARRANTY;
 This is free software, and you are welcome to redistribute it
 under certain conditions; see the LICENSE for details.</source>
-        <translation>Copyright (C) 2006-2021 Pedro Lopez-Cabanillas
+        <translation>Copyright (C) 2006-2022 Pedro Lopez-Cabanillas
 Este programa no tiene ABSOLUTAMENTE NINGUNA GARANTÍA;
 Esto es software libre; y usted puede redistribuirlo
-bajo ciertas condiciones; vea la LICENCIA para detalles.</translation>
+bajo ciertas condiciones; vea LICENSE para detalles.</translation>
     </message>
     <message>
         <source>Fatal error from the Operating System. This usually happens when the OS doesn&apos;t have MIDI support, or the MIDI support is not enabled. Please check your OS/MIDI configuration.</source>
@@ -1377,6 +1576,14 @@ bajo ciertas condiciones; vea la LICENCIA para detalles.</translation>
         <source>MIDI Out Connection.</source>
         <translation>Conexión de salida MIDI.</translation>
     </message>
+    <message>
+        <source>Input SMF/KAR/WRK file name.</source>
+        <translation>Nombre de archivo SMF/KAR/WRK de entrada.</translation>
+    </message>
+    <message>
+        <source>File not found:</source>
+        <translation>Archivo no encontrado:</translation>
+    </message>
 </context>
 <context>
     <name>RecentFilesHelper</name>
@@ -1389,4 +1596,82 @@ bajo ciertas condiciones; vea la LICENCIA para detalles.</translation>
         <translation>&amp;%1 %2</translation>
     </message>
 </context>
+<context>
+    <name>Sequence</name>
+    <message>
+        <source> in RIFF container of type %1</source>
+        <translation> en contenedor RIFF de tipo %1</translation>
+    </message>
+    <message>
+        <source>SMF type %1</source>
+        <translation>SMF tipo %1</translation>
+    </message>
+    <message>
+        <source>WRK file version v%1.%2</source>
+        <translation>versión de archivo WRK v%1.%2</translation>
+    </message>
+</context>
+<context>
+    <name>ToolBarEditDialog</name>
+    <message>
+        <source>Dialog</source>
+        <translation type="vanished">Diálogo</translation>
+    </message>
+    <message>
+        <source>remove</source>
+        <translation>eliminar</translation>
+    </message>
+    <message>
+        <source>add</source>
+        <translation>añadir</translation>
+    </message>
+    <message>
+        <source>up</source>
+        <translation>arriba</translation>
+    </message>
+    <message>
+        <source>down</source>
+        <translation>abajo</translation>
+    </message>
+    <message>
+        <source>Icon Only</source>
+        <translation>Solo icono</translation>
+    </message>
+    <message>
+        <source>Text Only</source>
+        <translation>Solo texto</translation>
+    </message>
+    <message>
+        <source>Text Beside Icon</source>
+        <translation>Texto junto a icono</translation>
+    </message>
+    <message>
+        <source>Text Under Icon</source>
+        <translation>Texto bajo icono</translation>
+    </message>
+    <message>
+        <source>Follow Style</source>
+        <translation>Seguir estilo</translation>
+    </message>
+    <message>
+        <source>Style of the Tool Buttons</source>
+        <translation>Estilo de los botones</translation>
+    </message>
+    <message>
+        <source>Available Actions</source>
+        <translation>Acciones disponibles</translation>
+    </message>
+    <message>
+        <source>Selected Actions</source>
+        <translation>Acciones seleccionadas</translation>
+    </message>
+    <message>
+        <source>Customize %1</source>
+        <translation>Personalizar %1</translation>
+    </message>
+    <message>
+        <source>Tool Bar Editor</source>
+        <translation>Editor de barra de herramientas</translation>
+    </message>
+</context>
 </TS>
